@@ -7,7 +7,6 @@
     with AT&T, you have an infringing copy of this software and cannot use
     it without violating AT&T's intellectual property rights.
 */
-#pragma prototyped
 
 #include <stdio.h>
 #include <aghdr.h>
@@ -114,7 +113,7 @@ char *agnameof(void *obj)
 {
 	Agraph_t	*g;
 	char		*rv;
-	static char buf[32];
+	char        buf[32];
 
 	/* perform internal lookup first */
 	g = agraphof(obj);
@@ -127,5 +126,5 @@ char *agnameof(void *obj)
 	}
 	if (AGTYPE(obj) != AGEDGE) sprintf(buf,"%c%ld",LOCALNAMEPREFIX,AGID(obj));
 	else buf[0] = 0;
-	return buf;
+	return agstrdup(g,buf);
 }
