@@ -1,5 +1,9 @@
+#ifdef __cplusplus
+extern "C" {
+#endif
+
 /*
-  $Id: generic_list.h,v 1.3 2002/09/11 18:59:46 erg Exp $
+  $Id: generic_list.h,v 1.5 2004/07/31 04:16:50 erg Exp $
 
   Copyright (c) 2002 Marcus Harnisch <marcus.harnisch@gmx.net>
 
@@ -19,9 +23,9 @@
 typedef void* gl_data;
 
 typedef struct generic_list_s {
-      unsigned long      used;  // number of elements in the list
-      unsigned long      size;  // number of elements that the list can hold
-      gl_data 		*data;  // pointer to first element
+      unsigned long      used;  /* number of elements in the list */
+      unsigned long      size;  /* number of elements that the list can hold */
+      gl_data 		*data;  /* pointer to first element */
 } generic_list_t;
 
 extern generic_list_t* new_generic_list(unsigned long size);
@@ -29,3 +33,7 @@ extern generic_list_t* add_to_generic_list(generic_list_t* list, gl_data element
 extern void            free_generic_list(generic_list_t* list);
 
 #endif /* GENERIC_LIST_H */
+
+#ifdef __cplusplus
+}
+#endif
