@@ -646,17 +646,14 @@ box_connection(node_t *n, pointf p)
 	
 	if (A_size < sides) {A_size = sides + 5; A = ALLOC(A_size,A,point);}
 
-	xsize = ((ND_lw_i(n) + ND_rw_i(n)) / ND_width(n)) * 16.0;
-	ysize = ((ND_ht_i(n)) / ND_height(n)) * 16.0;
+	xsize = (double)(ND_lw_i(n) + ND_rw_i(n)) / POINTS(ND_width(n));
+	ysize = (double)(ND_ht_i(n)) / POINTS(ND_height(n));
 
 	for (j = 0; j < peripheries; j++) {
 		for (i = 0; i < sides; i++) {
 			P = vertices[i+j*sides];
-/* simple rounding produces random results around .5 
- * this trick should clip off the random part. 
- * (note xsize/ysize prescaled by 16.0 above) */
-			A[i].x = ROUND(P.x * xsize) / 16;
-			A[i].y = ROUND(P.y * ysize) / 16;
+			A[i].x = ROUND(P.x * xsize);
+			A[i].y = ROUND(P.y * ysize);
 			if (sides > 2) {
 				A[i].x += ND_coord_i(n).x;
 				A[i].y += ND_coord_i(n).y;
