@@ -7,16 +7,12 @@
     with AT&T, you have an infringing copy of this software and cannot use
     it without violating AT&T's intellectual property rights.
 */
-#pragma prototyped
 
 /* TODO:
  *  Use encoded form for polyline and polygon
  */
 #include	"render.h"
-
-#ifdef DMALLOC
-#include "dmalloc.h"
-#endif
+#include	<ctype.h>
 
 #define SOLID  0
 #define DOTTED 1
@@ -193,7 +189,7 @@ static void set_color (Color* cp)
 
 static void initColors (void)
 {
-  colorlist = ALLOC(NPENS,0,Color);
+  colorlist = N_GNEW(NPENS,Color);
   colorlist[0] = white;
   colorlist[1] = black;
   ColorsUsed = 2;
@@ -266,7 +262,7 @@ static void set_line_style(int sty)
 static GC *makeGC(GC* old)
 {
   GC    *newGC;
-  newGC = ALLOC(1,0,GC);
+  newGC = GNEW(GC);
   if (old) *newGC = *old;
   else {
     newGC->bold = FALSE,
@@ -379,6 +375,13 @@ hpgl_begin_job(FILE *ofp, graph_t *g, char **lib, char *user, char *info[], poin
 {
 	/* Pages = pages; */
 	N_pages = pages.x * pages.y;
+}
+
+static void
+hpgl_begin_graph(GVC_t *gvc, graph_t* g, box bb, point pb)
+{
+    PB = bb;
+    PageWidth = pb.x;
     if (Output_lang == PCL) {
       prefix = pcl_prefix;
       suffix = pcl_suffix;
@@ -389,17 +392,6 @@ hpgl_begin_job(FILE *ofp, graph_t *g, char **lib, char *user, char *info[], poin
     }
 }
 
-static  void hpgl_end_job(void) { }
-
-static void
-hpgl_begin_graph(graph_t* g, box bb, point pb)
-{
-	PB = bb;
-    PageWidth = pb.x;
-}
-
-static void hpgl_end_graph(void) { }
-
 static void hpgl_set_scale (double scx,double scy)
 {
     char   buffer[64];
@@ -485,26 +477,6 @@ static void hpgl_end_page(void)
     destroyGC();
 }
 
-static void hpgl_begin_cluster(graph_t* g) { }
-
-static void hpgl_end_cluster (void) { }
-
-static void hpgl_begin_nodes(void) { }
-    
-static void hpgl_end_nodes(void) { }
-
-static void hpgl_begin_edges(void) { }            
-
-static void hpgl_end_edges(void) { }            
-
-static void hpgl_begin_node(node_t* n) { }
-
-static void hpgl_end_node (void) { }
-
-static void hpgl_begin_edge (edge_t* e) { }
-
-static void hpgl_end_edge (void) { }
-
 static void hpgl_begin_context(void)
 {
 	saveGC();
@@ -515,9 +487,9 @@ static void hpgl_end_context(void)
 	restoreGC();
 }
 
-static void mkFontCanon (char *old, char *new)
+static void mkFontCanon (unsigned char *old, unsigned char *new)
 {
-  char      c;
+  unsigned char      c;
   while ((c = *old++)) {
     if (isalnum(c) == FALSE) continue;
     if (isupper(c)) c = tolower(c);
@@ -569,7 +541,7 @@ static void mkFontInfo(char* name, double size, FontInfo* fip)
   char      buf[128];
   FontIndex *fi;
 
-  mkFontCanon(name,buf);
+  mkFontCanon((unsigned char*)name,(unsigned char*)buf);
   fi = fontIndex;
   for(i=0;i<sizeof(fontIndex)/sizeof(FontIndex)-1;i++) {
     if (streq(buf,fi->name)) break;
@@ -621,7 +593,7 @@ static void hpgl_set_style(char** s)
 		else if (streq(line,"filled")) { /* no-op */ }
 		else if (streq(line,"unfilled")) { /* no-op */ }
 		else {
-            fprintf(stderr, "hpgl_set_style: unsupported style %s - ignoring\n",
+            agerr(AGERR, "hpgl_set_style: unsupported style %s - ignoring\n",
                 line); 
         }
 	}
@@ -703,7 +675,7 @@ static void hpgl_bezier(point *A, int n, int arrow_at_start, int arrow_at_end)
 	int		j;
 
 	if (arrow_at_start || arrow_at_end)
-		fprintf(stderr,"hpgl_bezier illegal arrow args\n");
+		agerr(AGERR, "hpgl_bezier illegal arrow args\n");
     if (isInvis()) return;
 	sprintf(buffer,"PA%d,%d%sPD",CX(A[0].x),CY(A[0].y), Sep);
     output(buffer);
@@ -812,25 +784,32 @@ static void hpgl_polyline(point* A, int n)
 static void hpgl_user_shape(char *name, point *A, int n, int filled)
 {
 	static boolean onetime = TRUE;
-	if (onetime) {fprintf(stderr,"custom shapes not available with this driver\n"); onetime = FALSE;}
-	hpgl_polygon(A,n,filled);
+	if (onetime) {
+		agerr(AGERR, "custom shapes not available with this driver\n");
+		onetime = FALSE;
+	}
 }
 
 codegen_t	HPGL_CodeGen = {
 	hpgl_reset,
-	hpgl_begin_job, hpgl_end_job,
-	hpgl_begin_graph, hpgl_end_graph,
+	hpgl_begin_job, 0, /* hpgl_end_job */
+	hpgl_begin_graph, 0, /* hpgl_end_graph */
 	hpgl_begin_page, hpgl_end_page,
-	hpgl_begin_cluster, hpgl_end_cluster,
-	hpgl_begin_nodes, hpgl_end_nodes,
-	hpgl_begin_edges, hpgl_end_edges,
-	hpgl_begin_node, hpgl_end_node,
-	hpgl_begin_edge, hpgl_end_edge,
+	0, /* hpgl_begin_layer */ 0, /* hpgl_end_layer */
+	0, /* hpgl_begin_cluster */ 0, /* hpgl_end_cluster */
+	0, /* hpgl_begin_nodes */ 0, /* hpgl_end_nodes */
+	0, /* hpgl_begin_edges */ 0, /* hpgl_end_edges */
+	0, /* hpgl_begin_node */ 0, /* hpgl_end_node */
+	0, /* hpgl_begin_edge */ 0, /* hpgl_end_edge */
 	hpgl_begin_context, hpgl_end_context,
+	0, /* hpgl_begin_anchor */ 0, /* hpgl_end_anchor */
 	hpgl_set_font, hpgl_textline,
 	hpgl_set_color, hpgl_set_color, hpgl_set_style,
 	hpgl_ellipse, hpgl_polygon,
 	hpgl_bezier, hpgl_polyline,
-	0 /* hpgl_arrowhead */, hpgl_user_shape,
-	0 /* hpgl_comment */, 0 /* hpgl_textsize */
+	0, /* bezier_has_arrows */
+	0, /* hpgl_comment */
+	0, /* hpgl_textsize */
+	hpgl_user_shape,
+	0 /* hpgl_usershapesize */
 };
