@@ -485,36 +485,18 @@ portToTbl (htmltbl_t* tp, char *id)
 
 /* html_port:
  * See if edge port corresponds to part of the html node.
- * The initial ':' character has been stripped. 
- * If successful, fill in pp and return 1.
+ * Assume pname != "".
+ * If successful, return pointer to port's box.
+ * Else return NULL.
  */
-int
-html_port (node_t* n, char* pname, port* pp)
+box*
+html_port (node_t* n, char* pname)
 {
-  box*         bp;
-  char*        terminal;
   htmllabel_t* lbl = ND_label(n)->u.html;
 
-  if (lbl->kind == HTML_TEXT) return 0;
+  if (lbl->kind == HTML_TEXT) return NULL;
 
-	/* look for possible modifier port.{n,s,e,w} */
-  if ((terminal = strchr(pname,'.'))) {
-    char *buf = strdup(pname);
-    buf[terminal - pname] = 0;
-    pname = buf;		/* swap in properly terminated portname */
-    terminal++;
-  }
-  if ((bp = portToTbl (lbl->u.tbl, pname)) == NULL) {
-    if (terminal) free(pname);
-    return 0;
-  }
-
-  if (compassPort (n, bp, pp, terminal))
-    agerr(AGWARN, "node %s, port %s, unrecognized compass point '%c' - ignored\n",
-      n->name,pname,*terminal);
-
-  if (terminal) free(pname);
-  return 1;
+  return portToTbl (lbl->u.tbl, pname);
 }
 
 /* html_path:
