@@ -7,7 +7,6 @@
     with AT&T, you have an infringing copy of this software and cannot use
     it without violating AT&T's intellectual property rights.
 */
-#pragma prototyped
 /* mpgen.c 1999-Feb-23 Jim Hefferon jim@joshua.smcvt.edu 
  *  Adapted from psgen.c.  See 1st_read.mp.
  */
@@ -17,9 +16,6 @@
 #endif
 #include <sys/stat.h>
 
-#ifdef DMALLOC
-#include "dmalloc.h"
-#endif
 
 #define	NONE	0
 #define	NODE	1
@@ -81,7 +77,7 @@ mp_comment(void* obj, attrsym_t* sym)
 }
 
 static void
-mp_begin_graph(graph_t* g, box bb, point pb)
+mp_begin_graph(GVC_t *gvc, graph_t* g, box bb, point pb)
 {
 	/* PB = bb; */
 	if (onetime) {
@@ -93,11 +89,6 @@ mp_begin_graph(graph_t* g, box bb, point pb)
 	}
 }
 
-static void
-mp_end_graph(void)
-{
-}
-
 static void
 mp_begin_page(graph_t *g, point page, double scale, int rot, point offset)
 {
@@ -107,21 +98,6 @@ mp_begin_page(graph_t *g, point page, double scale, int rot, point offset)
         S[SP].size = 0.0;
 }
 
-static void mp_end_page(void) { }
-
-static void mp_begin_cluster(graph_t* g) { }
-
-static void mp_end_cluster(void) { }
-
-static void mp_begin_nodes(void) { }
-    
-static void mp_end_nodes(void) { }
-
-static void mp_begin_edges(void) { }            
-
-static void mp_end_edges(void) { }            
-
-
 static void
 mp_begin_node(node_t* n)
 {
@@ -129,8 +105,6 @@ mp_begin_node(node_t* n)
 	mp_comment(n,N_comment);
 }
 
-static void mp_end_node (void) { }
-
 static void
 mp_begin_edge (edge_t* e)
 {
@@ -138,19 +112,17 @@ mp_begin_edge (edge_t* e)
 	mp_comment(e,E_comment);
 }
 
-static void mp_end_edge (void) { }
-
 static void
 mp_begin_context(void)
 {
-	if (SP == STACKSIZE - 1) fprintf(stderr,"warning: mpgen stack overflow\n");
+	if (SP == STACKSIZE - 1) agerr(AGWARN, "mpgen stack overflow\n");
 	else {SP++; S[SP] = S[SP-1];}
 }
 
 static void
 mp_end_context(void)
 {
-	if (SP == 0) fprintf(stderr,"warning: mpgen stack underflow\n");
+	if (SP == 0) agerr(AGWARN, "mpgen stack underflow\n");
 	else SP--;
 }
 
@@ -203,14 +175,14 @@ mp_string(char *s)
 
         if (!buf) {
                 bufsize = 64;
-                buf = malloc(bufsize);
+                buf = N_GNEW(bufsize,char);
         }
 
 	p = buf;
 	while (*s)  {
 		if (pos > (bufsize-8)) {
 			bufsize *= 2;
-			buf = realloc(buf,bufsize);
+			buf = grealloc(buf,bufsize);
 			p = buf + pos;
 		}
 		if ((*s == LPAREN) || (*s == RPAREN)) {
@@ -236,7 +208,7 @@ mp_bezier(point *A, int n, int arrow_at_start, int arrow_at_end)
 {
 	int		j;
 	if (arrow_at_start || arrow_at_end)
-		fprintf(stderr," mp_bezier illegal arrow args\n");
+		agerr(AGERR, "mp_bezier illegal arrow args\n");
 	fprintf(Output_file,"draw (%dbp,%dbp) ",A[0].x,A[0].y);
 	for (j = 1; j < n; j += 3)
 		fprintf(Output_file,"\n  ..controls (%dbp,%dbp) and (%dbp,%dbp).. (%dbp,%dbp)",
@@ -290,22 +262,26 @@ mp_user_shape(char *name, point *A, int sides, int filled)
 	fprintf(Output_file,"]  %d %s %s ignored\n",sides,(filled?"true":"false"),name);
 }
 
-
 codegen_t	MP_CodeGen = {
 	mp_reset,
 	mp_begin_job, mp_end_job,
-	mp_begin_graph, mp_end_graph,
-	mp_begin_page, mp_end_page,
-	mp_begin_cluster, mp_end_cluster,
-	mp_begin_nodes, mp_end_nodes,
-	mp_begin_edges, mp_end_edges,
-	mp_begin_node, mp_end_node,
-	mp_begin_edge, mp_end_edge,
+	mp_begin_graph, 0, /* mp_end_graph */
+	mp_begin_page, 0, /* mp_end_page */
+	0, /* mp_begin_layer */ 0, /* mp_end_layer */
+	0, /* mp_begin_cluster */ 0, /* mp_end_cluster */
+	0, /* mp_begin_nodes */ 0, /* mp_end_nodes */
+	0, /* mp_begin_edges */ 0, /* mp_end_edges */
+	mp_begin_node, 0, /* mp_end_node */
+	mp_begin_edge, 0, /* mp_end_edge */
 	mp_begin_context, mp_end_context,
+	0, /* mp_begin_anchor */ 0, /* mp_end_anchor */
 	mp_set_font, mp_textline,
 	mp_set_color, mp_set_color, mp_set_style,
 	mp_ellipse, mp_polygon,
 	mp_bezier, mp_polyline,
-	0 /* mp_arrowhead */, mp_user_shape,
-	mp_comment, 0 /* mp_textsize */
+	0, /* bezier_has_arrows */
+	mp_comment,
+	0, /* mp_textsize */
+	mp_user_shape,
+	0 /* mp_usershapesize */
 };
