@@ -169,11 +169,14 @@ void dotneato_write_one(GVC_t *gvc)
 			attach_attrs(g);
 			agwrite(g,gvc->job->output_file); break;
 		case CANONICAL_DOT:
+			if (HAS_CLUST_EDGE(g)) undoClusterEdges(g);
 			agwrite(g,gvc->job->output_file); break;
 		case PLAIN:		
-			attach_attrs(g); write_plain(gvc,gvc->job->output_file); break;
+			/* attach_attrs(g);  */
+			write_plain(gvc,gvc->job->output_file); break;
 		case PLAIN_EXT:		
-			attach_attrs(g); write_plain_ext(gvc,gvc->job->output_file); break;
+			/* attach_attrs(g);  */
+			write_plain_ext(gvc,gvc->job->output_file); break;
 		default:
 			if (gvc->job->output_lang >= QBM_FIRST && gvc->job->output_lang < QBM_LAST)
 				emit_graph(gvc,chkOrder (g)); break;
@@ -245,6 +248,21 @@ static attrsym_t *safe_dcl(graph_t *g, void *obj, char *name, char *def,
 	return a;
 }
 
+/* setYInvert:
+ * Set parameters used to flip coordinate system (y=0 at top).
+ * Values do not need to be unset, since if y_invert is set, it's
+ * set for * all graphs during current run, so each will 
+ * reinitialize the values for its bbox.
+ */
+static void
+setYInvert (graph_t* g)
+{
+    if (y_invert) {
+		y_off = GD_bb(g).UR.y + GD_bb(g).LL.y;
+		yf_off = PS2INCH(y_off);
+	}
+}
+
 void attach_attrs(graph_t* g)
 {
 	int		i,j,sides;
@@ -256,10 +274,7 @@ void attach_attrs(graph_t* g)
 	point	pt;
 
 	e_arrows = s_arrows = 0;
-    if (y_invert) {
-		y_off = GD_bb(g).UR.y + GD_bb(g).LL.y;
-		yf_off = PS2INCH(y_off);
-	}
+	setYInvert (g);
     agxbinit (&xb, BUFSIZ, xbuffer);
 	safe_dcl(g,g->proto->n,"pos","",agnodeattr);
 	safe_dcl(g,g->proto->n,"rects","",agnodeattr);
@@ -310,7 +325,8 @@ void attach_attrs(graph_t* g)
 					if (i > 0) agxbputc (&xb, ' ');
 					if (poly->sides >= 3)
 						sprintf(buf,"%.3f %.3f",
-							poly->vertices[i].x,YF(poly->vertices[i].y));
+							PS2INCH(poly->vertices[i].x),
+							YF(PS2INCH(poly->vertices[i].y)));
 					else
 						sprintf(buf,"%.3f %.3f",
 						  ND_width(n)/2.0 * cos(i/(double)sides * PI * 2.0),
@@ -366,6 +382,9 @@ void attach_attrs(graph_t* g)
 	}
 	rec_attach_bb(g);
     agxbfree (&xb);
+
+	if (HAS_CLUST_EDGE(g)) 
+		undoClusterEdges(g);
 }
 
 void rec_attach_bb(graph_t* g)
@@ -405,9 +424,12 @@ static char *canonical(char *str)
 	return agstrcanon(str,getoutputbuffer(str));
 }
 
-static void writenodeandport(FILE *fp, char *node, char *port)
+static void writenodeandport(FILE *fp, node_t *node, char *port)
 {
-	fprintf(fp,"%s",canonical(node));       /* slimey i know*/
+	char* name;
+ 	if (IS_CLUST_NODE(node)) name = strchr(node->name,':')+1;
+	else name = node->name;
+	fprintf(fp,"%s",canonical(name));       /* slimey i know*/
 	if (port && *port) fprintf(fp,"%c%s",port[0],canonical(port+1));
 }
 
@@ -417,11 +439,6 @@ static void writenodeandport(FILE *fp, char *node, char *port)
 #define HEADX 2
 
 /* _write_plain:
- * Assumes y_invert parameters have been set.
- * At present, this is done by a previous call to attach_attrs.
- * As this appears to be the only reason to call attach_attrs for plain
- * output, it may be cleaner to remove that call and handle y_invert
- * here.
  */
 void _write_plain(GVC_t *gvc, FILE* f, boolean extend)
 {
@@ -435,10 +452,12 @@ void _write_plain(GVC_t *gvc, FILE* f, boolean extend)
 	char*       lbl;
 
 	setup_graph(gvc, g);
+	setYInvert (g);
 	pt = GD_bb(g).UR;
 	fprintf(f,"graph %.3f %.3f %.3f\n",
 		GD_drawing(g)->scale, PS2INCH(pt.x), PS2INCH(pt.y));
 	for (n = agfstnode(g); n; n = agnxtnode(g,n)) {
+ 		if (IS_CLUST_NODE(n)) continue;
 		fprintf(f,"node %s ",canonical(n->name)); printptf(f,ND_coord_i(n));
 		lbl = ND_label(n)->text;
 		if (lbl) lbl = canonical(lbl);
@@ -464,9 +483,9 @@ void _write_plain(GVC_t *gvc, FILE* f, boolean extend)
 					splinePoints += bz.size;
 				}
 				fprintf(f,"edge ");
-				writenodeandport(f,e->tail->name,tport);
+				writenodeandport(f,e->tail,tport);
 				fprintf(f," ");
-				writenodeandport(f,e->head->name,hport);
+				writenodeandport(f,e->head,hport);
 				fprintf(f," %d",splinePoints);
 				for (i = 0; i < ED_spl(e)->size; i++) {
 					bz = ED_spl(e)->list[i];
