@@ -7,7 +7,6 @@
     with AT&T, you have an infringing copy of this software and cannot use
     it without violating AT&T's intellectual property rights.
 */
-#pragma prototyped
 
 /*
     J$: added `pdfmark' URL embedding.  PostScript rendered from
@@ -17,20 +16,23 @@
 #define	PDFMAX	3240	/*  Maximum size of Distiller's PDF canvas  */
 
 #include	"render.h"
+#include	"gvrender.h"
 #include	"ps.h"
+#include	"utils.h"
+
 #ifndef MSWIN32
 #include <unistd.h>
 #endif
-#include <sys/stat.h>
 
-#ifdef DMALLOC
-#include "dmalloc.h"
-#endif
+#include <sys/stat.h>
+#include <stdio.h>
+
 
 static	int		N_pages,Cur_page;
 /* static 	point	Pages; */
 static	box		PB;
 static int		onetime = TRUE;
+static node_t*  Curnode;  /* for user shapes */
 
 static char	*Fill = "fill\n";
 static char	*Stroke = "stroke\n";
@@ -88,8 +90,200 @@ ps_comment(void* obj, attrsym_t* sym)
 	if (str[0]) fprintf(Output_file,"%% %s\n",str);
 }
 
+static int  N_EPSF_files;
+static Dict_t	*EPSF_contents;
+
+typedef struct epsf_s {
+    int     macro_id;
+    point   offset;
+} epsf_t;
+
+typedef struct {
+    Dtlink_t        link;
+    char            *name;
+	int				macro_id;
+	point			size;
+	point			origin;
+    char            *contents;
+} ps_image_t;
+
+static void ps_image_free(Dict_t *dict, ps_image_t* p, Dtdisc_t *disc)
+{
+	free (p->contents);
+}
+
+static Dtdisc_t ImageDictDisc = {
+    offsetof(ps_image_t,name),   /* key */
+    -1,                     /* size */
+    0,                      /* link offset */
+    NIL(Dtmake_f),
+	(Dtfree_f)ps_image_free,
+    NIL(Dtcompar_f),
+    NIL(Dthash_f),
+    NIL(Dtmemory_f),
+    NIL(Dtevent_f)
+};
+
+static ps_image_t*
+user_init(char* str)
+{
+	char	*contents;
+	char	line[BUFSIZ];
+	FILE	*fp;
+	struct stat statbuf;
+	int		saw_bb;
+	int		lx,ly,ux,uy;
+	ps_image_t	*val;
+	
+	if (!EPSF_contents) EPSF_contents = dtopen(&ImageDictDisc,Dtoset);
+
+	val = dtsearch(EPSF_contents,str);
+	if (val) return val;
+
+	if (!(fp = fopen(str,"r"))) {
+		agerr(AGWARN, "couldn't open epsf file %s\n",str);
+		return NULL;
+	}
+		/* try to find size */
+	saw_bb = FALSE;
+	while (fgets(line, sizeof(line), fp)) {
+		if (sscanf(line,"%%%%BoundingBox: %d %d %d %d",&lx,&ly,&ux,&uy) == 4) {
+			saw_bb = TRUE;
+			break;
+		}
+	}
+
+	if (saw_bb) {
+		val = GNEW(ps_image_t);
+		val->origin.x = lx;
+		val->origin.y = ly;
+		val->size.x = ux - lx;
+		val->size.y = uy - ly;
+		val->name = str;
+		val->macro_id = N_EPSF_files++;
+		fstat(fileno(fp),&statbuf);
+		contents = val->contents = N_GNEW(statbuf.st_size+1,char);
+		fseek(fp,0,SEEK_SET);
+		fread(contents,statbuf.st_size,1,fp);
+		contents[statbuf.st_size] = '\0';
+		fclose(fp);
+		dtinsert(EPSF_contents,val);
+		return val;
+	}
+	else {
+		agerr(AGWARN, "BoundingBox not found in epsf file %s\n",str);
+		return NULL;
+	}
+}
+
+void epsf_init(node_t *n)
+{
+	epsf_t	*desc;
+	char*   str;
+	ps_image_t* img;
+	int     dx, dy;
+
+	if ((str=safefile(agget(n,"shapefile")))) {
+		img = user_init(str);
+		if (!img) return;
+		dx = img->size.x;
+		dy = img->size.y;
+		ND_width(n) = PS2INCH(dx);
+		ND_height(n) = PS2INCH(dy);
+		ND_shape_info(n) = desc = NEW(epsf_t);
+		desc->macro_id = img->macro_id;
+		desc->offset.x = -img->origin.x - (dx)/2;
+		desc->offset.y = -img->origin.y - (dy)/2;
+	}
+	else agerr(AGWARN, "shapefile not set for epsf node %s\n",n->name);
+}
+
+void epsf_free(node_t *n)
+{
+
+	if (ND_shape_info(n))
+		free(ND_shape_info(n));
+}
+
+void epsf_gencode(GVC_t *gvc, node_t *n)
+{
+	epsf_t	*desc;
+
+	gvc->n = n;
+
+	desc = (epsf_t*)(ND_shape_info(n));
+	if (!desc) return;
+	gvrender_begin_context(gvc);
+	if (desc) fprintf(Output_file,"%d %d translate newpath user_shape_%d\n",
+		ND_coord_i(n).x+desc->offset.x,ND_coord_i(n).y+desc->offset.y,
+		desc->macro_id);
+	ND_label(n)->p = ND_coord_i(n);
+	gvrender_end_context(gvc);
+	emit_label(gvc, ND_label(n), (void*)n);
+}
+
 static void
-ps_begin_graph(graph_t* g, box bb, point pb)
+epsf_define(void)
+{
+#define FILTER_EPSF 1	
+#if FILTER_EPSF
+	char	*p;
+#endif
+	ps_image_t*   img;
+
+	if (!EPSF_contents) return;
+	for (img = dtfirst(EPSF_contents); img; img = dtnext(EPSF_contents,img)) {
+		fprintf(Output_file,"/user_shape_%d {\n",img->macro_id);
+
+		if (fputs("%%BeginDocument:\n", Output_file) == EOF) {
+			perror("epsf_define()->fputs");
+			exit(EXIT_FAILURE);
+		}
+
+#if FILTER_EPSF
+		/* this removes EPSF DSC comments that, when nested in another
+		 * document, cause errors in Ghostview and other Postscript
+		 * processors (although legal according to the Adobe EPSF spec).
+		 */
+		p = img->contents;
+		while (*p) {	/* skip %%EOF lines */
+			if ((p[0] == '%') && (p[1] == '%') && (
+					!strncasecmp(&p[2],"EOF",3) ||
+					!strncasecmp(&p[2],"BEGIN",5) ||
+					!strncasecmp(&p[2],"END",3) ||
+					!strncasecmp(&p[2],"TRAILER",7)
+						)) {
+		/* check for *p since last line might not end in '\n' */
+				while (*p && (*p++ != '\n'));  
+				continue;
+			}
+			do {fputc(*p,Output_file);} while (*p++ != '\n');
+		}
+#else
+		if (fputs(img->contents, Output_file) == EOF) {
+			perror("epsf_define()->fputs");
+			exit(EXIT_FAILURE);
+		}
+#endif
+
+		if (fputs("%%EndDocument\n", Output_file) == EOF) {
+			perror("epsf_define()->fputs");
+			exit(EXIT_FAILURE);
+		}
+
+		if (fputs("} bind def\n",Output_file) == EOF) {
+			perror("epsf_define()->fputs");
+			exit(EXIT_FAILURE);
+		}
+
+#if 0
+		fprintf(Output_file,"} bind def\n");
+#endif
+	}
+}
+
+static void
+ps_begin_graph(GVC_t *gvc, graph_t* g, box bb, point pb)
 {
 	char *s;
 	static char setupLatin1 = FALSE;
@@ -104,11 +298,11 @@ ps_begin_graph(graph_t* g, box bb, point pb)
 		epsf_define();
 
  		/*  Set base URL for relative links (for Distiller >= 3.0)  */
- 		if ((s = agget(g, "URL")) && strlen(s)) {
+ 		if (((s = agget(g, "href")) && s[0]) 
+ 		  || ((s = agget(g, "URL")) && s[0])) {
  			fprintf(Output_file,
 				"[ {Catalog} << /URI << /Base (%s) >> >>\n"
  				"/PUT pdfmark\n", s);
-
 		}
 	}
 	if (GD_has_Latin1char(g) && !setupLatin1) {
@@ -120,6 +314,10 @@ ps_begin_graph(graph_t* g, box bb, point pb)
 static void
 ps_end_graph(void)
 {
+	if (EPSF_contents) {
+		dtclose(EPSF_contents); EPSF_contents = 0;
+		N_EPSF_files = 0;
+	}
 	onetime = FALSE;
 }
 
@@ -150,10 +348,10 @@ ps_begin_page(graph_t *g, point page, double scale, int rot, point offset)
  	/*  Define the size of the PS canvas  */
 	if (Output_lang == PDF) {
  		if (PB.UR.x >= PDFMAX || PB.UR.y >= PDFMAX)
-			fprintf(stderr,
-			 "%s: warning, canvas size (%d,%d) exceeds PDF limit (%d)\n"
+			agerr(AGWARN,
+			 "canvas size (%d,%d) exceeds PDF limit (%d)\n"
  				"\t(suggest setting a bounding box size, see dot(1))\n",
- 				CmdName, PB.UR.x, PB.UR.y, PDFMAX);
+ 				PB.UR.x, PB.UR.y, PDFMAX);
  		fprintf(Output_file,"[ /CropBox [%d %d %d %d] /PAGES pdfmark\n",
  			PB.LL.x, PB.LL.y, PB.UR.x+1, PB.UR.y+1);
 	}
@@ -162,12 +360,20 @@ ps_begin_page(graph_t *g, point page, double scale, int rot, point offset)
 static void
 ps_end_page(void)
 {
-	fprintf(Output_file,"endpage\ngrestore\n");
+	/* the showpage is really a no-op, but at least one PS processor
+	 * out there needs to see this literal token.  endpage does the real work.
+	 */
+	fprintf(Output_file,"endpage\nshowpage\ngrestore\n");
 	fprintf(Output_file,"%%%%PageTrailer\n");
 	fprintf(Output_file,"%%%%EndPage: %d\n",Cur_page);
 	assert(SP == 0);
 }
 
+static void ps_begin_layer(char *s, int n, int Nlayers) 
+{ 
+	fprintf(Output_file,"%d %d setlayer\n", n, Nlayers);
+}
+    
 static void
 ps_begin_cluster(graph_t* g)
 {
@@ -177,19 +383,10 @@ ps_begin_cluster(graph_t* g)
 	map_begin_cluster(g);
 }
 
-static void ps_end_cluster(void) { }
-
-static void ps_begin_nodes(void) { }
-    
-static void ps_end_nodes(void) { }
-
-static void ps_begin_edges(void) { }            
-
-static void ps_end_edges(void) { }            
-
 static void
 ps_begin_node(node_t* n)
 {
+	Curnode = n;
 	fprintf(Output_file,"\n%%\t%s\n",n->name);
 	ps_comment(n,N_comment);
 
@@ -197,8 +394,6 @@ ps_begin_node(node_t* n)
 	map_begin_node(n);
 }
 
-static void ps_end_node (void) { }
-
 static void
 ps_begin_edge (edge_t* e)
 {
@@ -209,20 +404,19 @@ ps_begin_edge (edge_t* e)
     map_begin_edge(e);
 }
 
-static void ps_end_edge (void) { }
 
 static void
 ps_begin_context(void)
 {
 	fprintf(Output_file,"gsave 10 dict begin\n");
-	if (SP == STACKSIZE - 1) fprintf(stderr,"warning: psgen stk ovfl\n");
+	if (SP == STACKSIZE - 1) agerr(AGWARN, "psgen stk ovfl\n");
 	else {SP++; S[SP] = S[SP-1];}
 }
 
 static void
 ps_end_context(void)
 {
-	if (SP == 0) fprintf(stderr,"warning: psgen stk undfl\n");
+	if (SP == 0) agerr(AGWARN, "psgen stk undfl\n");
 	else SP--;
 	fprintf(Output_file,"end grestore\n");
 }
@@ -251,7 +445,8 @@ ps_set_color(char* name)
 static void
 ps_set_pencolor(char* name)
 {
-	if (strcmp(name,S[SP].pencolor)) {
+	if (streq(name,"transparent")) S[SP].pencolor = "";
+	else if (strcmp(name,S[SP].pencolor)) {
 		ps_set_color(name);   /* change pen color immediately */
 		S[SP].pencolor = name;
 	}
@@ -260,7 +455,9 @@ ps_set_pencolor(char* name)
 static void
 ps_set_fillcolor(char* name)
 {
-	S[SP].fillcolor = name;      /* defer changes to fill color to shape */
+      /* defer changes to fill color to shape */
+	if (streq(name,"transparent")) S[SP].fillcolor = "";
+	else S[SP].fillcolor = name;
 }
 
 static void
@@ -288,7 +485,7 @@ ps_string(char *s)
 
         if (!buf) {
                 bufsize = 64;
-                buf = malloc(bufsize);
+                buf = N_GNEW(bufsize,char);
         }
 
 	p = buf;
@@ -297,7 +494,7 @@ ps_string(char *s)
 	while (*s)  {
 		if (pos > (bufsize-8)) {
 			bufsize *= 2;
-			buf = realloc(buf,bufsize);
+			buf = grealloc(buf,bufsize);
 			p = buf + pos;
 		}
 		if ((*s == LPAREN) || (*s == RPAREN) || (*s == '\\')) {
@@ -317,28 +514,47 @@ ps_textline(point p, textline_t *line)
 {
 	double adj;
 
-	switch(line->just) {
-		case 'l':
-			adj = 0.0;
-			break;
-		case 'r':
-			adj = -1.0;
-			break;
-		default:
-		case 'n':
-			adj = -0.5;
-			break;
+	if (*S[SP].pencolor == '\0') return;
+	if (line->xshow) {
+		switch(line->just) {
+			case 'l':
+				break;
+			case 'r':
+				p.x -= line->width;
+				break;
+			default:
+			case 'n':
+				p.x -= line->width / 2;
+				break;
+		}
+		fprintf(Output_file,"%d %d moveto\n%s\n[%s]\nxshow\n",
+			p.x,p.y,ps_string(line->str), line->xshow);
+	}
+	else {
+		switch(line->just) {
+			case 'l':
+				adj = 0.0;
+				break;
+			case 'r':
+				adj = -1.0;
+				break;
+			default:
+			case 'n':
+				adj = -0.5;
+				break;
+		}
+		fprintf(Output_file,"%d %d moveto %.1f %.1f %s alignedtext\n",
+			p.x,p.y,line->width,adj,ps_string(line->str));
 	}
-	fprintf(Output_file,"%d %d moveto %d %.1f %s alignedtext\n",
-		p.x,p.y,line->width,adj,ps_string(line->str));
 }
 
 static void
 ps_bezier(point *A, int n, int arrow_at_start, int arrow_at_end)
 {
 	int		j;
+	if (*S[SP].pencolor == '\0') return;
 	if (arrow_at_start || arrow_at_end)
-		fprintf(stderr,"ps_bezier illegal arrow args\n");
+		agerr(AGERR, "ps_bezier illegal arrow args\n");
 	fprintf(Output_file,Newpath_Moveto,A[0].x,A[0].y);
 	for (j = 1; j < n; j += 3)
 		fprintf(Output_file,"%d %d %d %d %d %d curveto\n",
@@ -351,33 +567,33 @@ ps_polygon(point *A, int n, int filled)
 {
 	int		j;
 
-	if (filled) {
+	if (filled && *S[SP].fillcolor) {
 		ps_set_color(S[SP].fillcolor);
 		fprintf(Output_file,Newpath_Moveto,A[0].x,A[0].y);
 		for (j = 1; j < n; j++)
 		       	fprintf(Output_file,"%d %d lineto\n",A[j].x,A[j].y);
 		fprintf(Output_file,"closepath\n");
 		fprintf(Output_file, Fill);
-		ps_set_color(S[SP].pencolor);
-	}
-	if (!filled || (filled && strcmp(S[SP].fillcolor,S[SP].pencolor))) {
-		fprintf(Output_file,Newpath_Moveto,A[0].x,A[0].y);
-		for (j = 1; j < n; j++)
-		       	fprintf(Output_file,"%d %d lineto\n",A[j].x,A[j].y);
-		fprintf(Output_file,"closepath\n");
-		fprintf(Output_file, Stroke);
+		if (*S[SP].pencolor) ps_set_color(S[SP].pencolor);
 	}
+	if (*S[SP].pencolor == '\0') return;
+	fprintf(Output_file,Newpath_Moveto,A[0].x,A[0].y);
+	for (j = 1; j < n; j++)
+	       	fprintf(Output_file,"%d %d lineto\n",A[j].x,A[j].y);
+	fprintf(Output_file,"closepath\n");
+	fprintf(Output_file, Stroke);
 }
 
 static void
 ps_ellipse(point p, int rx, int ry, int filled)
 {
-	if (filled) {
+	if (filled && *S[SP].fillcolor) {
 		ps_set_color(S[SP].fillcolor);
 		fprintf(Output_file,"%d %d %d %d ellipse_path\n",p.x,p.y,rx,ry);
 		fprintf(Output_file, Fill);
-		ps_set_color(S[SP].pencolor);
+		if (*S[SP].pencolor) ps_set_color(S[SP].pencolor);
 	}
+	if (*S[SP].pencolor == '\0') return;
 	if (!filled || (filled && strcmp(S[SP].fillcolor,S[SP].pencolor))) {
 		fprintf(Output_file,"%d %d %d %d ellipse_path\n",p.x,p.y,rx,ry);
 		fprintf(Output_file, Stroke);
@@ -389,6 +605,7 @@ ps_polyline(point* A, int n)
 {
 	int		j;
 
+	if (*S[SP].pencolor == '\0') return;
 	fprintf(Output_file,Newpath_Moveto,A[0].x,A[0].y);
 	for (j = 1; j < n; j ++) fprintf(Output_file,"%d %d lineto\n",A[j].x,A[j].y);
 	fprintf(Output_file,Stroke);
@@ -397,107 +614,63 @@ ps_polyline(point* A, int n)
 static void
 ps_user_shape(char *name, point *A, int sides, int filled)
 {
-	int		j;
-	fprintf(Output_file,"[ ");
-	for (j = 0; j < sides; j++) fprintf(Output_file,"%d %d ",A[j].x,A[j].y);
-	fprintf(Output_file,"%d %d ",A[0].x,A[0].y);
-	fprintf(Output_file,"]  %d %s %s\n",sides,(filled?"true":"false"),name);
-}
+	int		    j;
+	ps_image_t* img;
+	point       offset;
+	char*       shapeimagefile;
+	char*       suffix;
 
-#define N_EPSF 32
-static int  N_EPSF_files;
-static char *EPSF_contents[N_EPSF];
-
-typedef struct epsf_s {
-	int		macro_id;
-	point	offset;
-} epsf_t;
-
-void epsf_init(node_t* n)
-{
-	char	*str,*contents;
-	char	line[BUFSIZ];
-	FILE	*fp;
-	struct stat statbuf;
-	int		i, saw_bb;
-	int		lx,ly,ux,uy;
-	epsf_t	*desc;
-
-	str = agget(n,"shapefile");
-	if (str && str[0]) {
-		if ((fp = fopen(str,"r"))) {
-			/* try to find size */
-			saw_bb = FALSE;
-			while (fgets(line, sizeof(line), fp)) {
-			  if (sscanf(line,"%%%%BoundingBox: %d %d %d %d",&lx,&ly,&ux,&uy) == 4) {
-				saw_bb = TRUE;
-				break;
-			  }
-			}
-
-			if (saw_bb) {
-				ND_width(n) = PS2INCH(ux - lx);
-				ND_height(n) = PS2INCH(uy - ly);
-				fstat(fileno(fp),&statbuf);
-				i = N_EPSF_files++;
-				ND_shape_info(n) = desc = NEW(epsf_t);
-				desc->macro_id = i;
-				desc->offset.x = -lx - (ux - lx)/2;
-				desc->offset.y = -ly - (uy - ly)/2;
-				contents = EPSF_contents[i] = malloc(statbuf.st_size+1);
-				fseek(fp,0,SEEK_SET);
-				fread(contents,statbuf.st_size,1,fp);
-				contents[statbuf.st_size] = '\0';
-				fclose(fp);
-			}
-			else fprintf(stderr, "%s: warning, BoundingBox not found in epsf file %s\n",CmdName,str);
+	if (streq(name,"custom")) {
+		shapeimagefile = agget(Curnode,"shapefile");
+		if (EPSF_contents && (img = dtmatch(EPSF_contents, shapeimagefile))) {
+			ps_begin_context();
+			offset.x = -img->origin.x - (img->size.x)/2;
+			offset.y = -img->origin.y - (img->size.y)/2;
+			fprintf(Output_file,"%d %d translate newpath user_shape_%d\n",
+				ND_coord_i(Curnode).x+offset.x,ND_coord_i(Curnode).y+offset.y,
+				img->macro_id);
+			ps_end_context();
 		}
-		else fprintf(stderr," %s: warning, couldn't open epsf file %s\n",CmdName,str);
-	}
-	else fprintf(stderr, "%s: warning, shapefile not set for epsf node %s\n",CmdName,n->name);
-}
-
-void epsf_free(node_t* n)
-{
-/* FIXME  What about the allocated EPSF_contents[i] ? */
-
-	if (ND_shape_info(n)) free(ND_shape_info(n));
-}
-
-void epsf_gencode(node_t *n)
-{
-	epsf_t	*desc;
-
-	desc = (epsf_t*)(ND_shape_info(n));
-	if (!desc) return;
-	CodeGen->begin_node(n);
-	CodeGen->begin_context();
-	if (desc) fprintf(Output_file,"%d %d translate newpath user_shape_%d\n",
-		ND_coord(n).x+desc->offset.x,ND_coord(n).y+desc->offset.y,
-		desc->macro_id);
-	CodeGen->end_context();
-	CodeGen->end_node();
-}
-
-void epsf_define(void)
-{
-	int		i;
-	char	*p;
-
-	for (i = 0; i < N_EPSF_files; i++) {
-		fprintf(Output_file,"/user_shape_%d {",i);
-		p = EPSF_contents[i];
-		while (*p) {
-			if ((p[0] == '%') && (p[1] == '%') && (p[1] != '!')) {
-				while (*p++ != '\n');
-				continue;
+		else {
+			suffix = strrchr(shapeimagefile,'.');
+			if (suffix) {
+				suffix++;
+				if (strcmp(suffix, "ps")) { 
+					agerr(AGERR, "image type \"%s\" not supported in PostScript output\n",suffix);
+				}
+				else {
+					agerr(AGERR, "Could not find image file \"%s\"\n",shapeimagefile);
+				}
+			}
+			else {
+				agerr(AGERR, "image file %s not supported in PostScript output\n",shapeimagefile);
 			}
-			do {fputc(*p,Output_file);} while (*p++ != '\n');
 		}
-		free(EPSF_contents[i]);
-		fprintf(Output_file,"} bind def\n");
 	}
-	N_EPSF_files = 0;
+	else {
+		fprintf(Output_file,"[ ");
+		for (j = 0; j < sides; j++) fprintf(Output_file,"%d %d ",A[j].x,A[j].y);
+		fprintf(Output_file,"%d %d ",A[0].x,A[0].y);
+		fprintf(Output_file,"]  %d %s %s\n",sides,(filled?"true":"false"),name);
+	}
+}
+
+point ps_user_shape_size(node_t *n, char *shapeimagefile)
+{
+	point	rv;
+	ps_image_t* img;
+
+	rv.x = rv.y = -1;  /* assume error */
+	if (shapeimagefile && *shapeimagefile) { 
+		img = user_init (shapeimagefile);
+		if (img) {
+			rv.x = img->size.x;
+			rv.y = img->size.y;
+		}
+	}
+	else
+		rv.x = rv.y = 0;
+	return rv;
 }
 
 codegen_t	PS_CodeGen = {
@@ -505,16 +678,21 @@ codegen_t	PS_CodeGen = {
 	ps_begin_job, ps_end_job,
 	ps_begin_graph, ps_end_graph,
 	ps_begin_page, ps_end_page,
-	ps_begin_cluster, ps_end_cluster,
-	ps_begin_nodes, ps_end_nodes,
-	ps_begin_edges, ps_end_edges,
-	ps_begin_node, ps_end_node,
-	ps_begin_edge, ps_end_edge,
+	ps_begin_layer, 0, /* ps_end_layer */
+	ps_begin_cluster, 0, /* ps_end_cluster */
+	0, /* ps_begin_nodes */ 0, /* ps_end_nodes */
+	0, /* ps_begin_edges */ 0, /* ps_end_edges */
+	ps_begin_node, 0, /* ps_end_node */
+	ps_begin_edge, 0, /* ps_end_edge */
 	ps_begin_context, ps_end_context,
+	0, /* ps_begin_anchor */ 0, /* ps_end_anchor */
 	ps_set_font, ps_textline,
 	ps_set_pencolor, ps_set_fillcolor, ps_set_style,
 	ps_ellipse, ps_polygon,
 	ps_bezier, ps_polyline,
-	0 /* ps_arrowhead */, ps_user_shape,
-	ps_comment, 0 /* ps_textsize */
+	0, /* bezier_has_arrows */
+	ps_comment,
+	0, /* ps_textsize */
+	ps_user_shape,
+	0 /* usershapesize */
 };
