@@ -40,13 +40,13 @@ static shape_desc* point_desc;
 /* forward declarations of functions used in shapes tables */
 
 static void  poly_free(node_t *n);
-static port	poly_port(node_t *n, char *portname);
+static port	poly_port(node_t *n, char *portname, char*);
 static boolean  poly_inside(inside_t *inside_context, pointf p);
 static int   poly_path(node_t *n, edge_t *e, int pt, box rv[], int* kptr);
 static void  poly_gencode(GVC_t *gvc, node_t *n);
 
 static void  record_free(node_t *n);
-static port  record_port(node_t *n, char *portname);
+static port  record_port(node_t *n, char *portname, char*);
 static boolean record_inside(inside_t *inside_context, pointf p);
 static int   record_path(node_t *n, edge_t *e, int pt, box rv[], int* kptr);
 static void  record_gencode(GVC_t *gvc, node_t *n);
@@ -772,40 +772,91 @@ static int   poly_path(node_t *n, edge_t *e, int pt, box rv[], int* kptr)
 	return side;
 }
 
-static port poly_port(node_t *n, char* portname)
+/* compassPort:
+ * Attach a compass point to a port pp, and fill in remaining fields.
+ * n is the corresponding node; bp is the bounding box of the port.
+ * compass is the compass point
+ * Return 1 if unrecognized compass point, in which case we
+ * use the center.
+ */
+static int
+compassPort (node_t* n, box* bp, port* pp, char* compass)
 {
-	static char *points_of_compass[] =
-		{"n","ne","e","se","s","sw","w","nw",NULL};
-static struct {signed char x,y;} a[] = {{0,1},{1,1},{1,0},{1,-1},{0,-1},{-1,-1},{-1,0},{-1,1}};
+	box   b;
+	point p, ctr;
+	int   rv = 0;
 
-	int		i,ht,wd;
-	port	rv;
-	char	*p;
-
-	if (portname[0] != ':') return Center;		/*could be '\000' */
-	portname++; /* skip over delim */
-
-	if (ND_label(n)->html) {
-		if (html_port (n, portname, &rv)) return rv;
-	}
-
-	for (i = 0; (p = points_of_compass[i]); i++)
-		if (streq(p,portname)) break;
-
-	if (p == NULL) {
-		unrecognized(n,portname);
-		rv = Center;
+	if (bp) {
+		b = *bp;
+		p = pointof((b.LL.x+b.UR.x)/2,(b.LL.y+b.UR.y)/2);
 	}
 	else {
-		ht = ND_ht_i(n) / 2; 
-		wd = ND_lw_i(n);
-		rv.p.x = a[i].x * wd;
-		rv.p.y = a[i].y * ht;
-		rv.bp = 0;
-		rv.order = (MC_SCALE * (ND_lw_i(n) + rv.p.x)) / (ND_lw_i(n) + ND_rw_i(n));
-		rv.constrained = FALSE;
-		rv.defined = TRUE;
+		p.x = p.y = 0;
+		if (GD_flip(n->graph)) {
+			b.UR.x =  ND_ht_i(n) / 2; 
+			b.LL.x =  -b.UR.x;
+			b.UR.y =  ND_lw_i(n);
+			b.LL.y =  -b.UR.y;
+		}
+		else {
+			b.UR.y =  ND_ht_i(n) / 2; 
+			b.LL.y =  -b.UR.y;
+			b.UR.x =  ND_lw_i(n);
+			b.LL.x =  -b.UR.x;
+		}
 	}
+	ctr = p;
+	if (compass) {
+		int margin = 1;	/* would like = 0, but spline router croaks */
+		switch(*compass++) {
+		case 'e':	p.x = b.UR.x - margin; break;
+		case 's':	
+			p.y = b.LL.y + margin; break;
+			switch(*compass) {
+			case '\0':	break;
+			case 'e':	p.x = b.UR.x - margin; break;
+			case 'w':	p.x = b.LL.x + margin; break;
+			default :   p.y = ctr.y; rv = 1; break;
+			}
+			break;
+		case 'w':	p.x = b.LL.x + margin; break;
+		case 'n':	
+			p.y = b.UR.y - margin; 
+			switch(*compass) {
+			case '\0':	break;
+			case 'e':	p.x = b.UR.x - margin; break;
+			case 'w':	p.x = b.LL.x + margin; break;
+			default :   p.y = ctr.y; rv = 1; break;
+			}
+			break;
+		default :   rv = 1; break;
+		}
+	}
+	p = invflip_pt(p, GD_rankdir(n->graph));
+	pp->bp = bp;
+	pp->p = p;
+	pp->order = (MC_SCALE * (ND_lw_i(n) + p.x)) / (ND_lw_i(n) + ND_rw_i(n));
+	pp->constrained = FALSE;
+	pp->defined = TRUE;
+	return rv;
+}
+
+static port poly_port(node_t *n, char* portname, char* compass)
+{
+	port	rv;
+	box*    bp;
+
+	if (portname[0] == '\0') return Center;
+
+	if ((ND_label(n)->html) && ((bp = html_port (n, portname)))) {
+		if (compassPort (n, bp, &rv, compass))
+			agerr(AGWARN, "node %s, port %s, unrecognized compass point '%s' - ignored\n",
+			n->name,portname,compass);
+	}
+	else if (compassPort (n, NULL, &rv, portname)) {
+		unrecognized(n,portname);
+	}
+
 	return rv;
 }
 
@@ -827,9 +878,8 @@ static void poly_gencode(GVC_t *gvc, node_t *n)
 	if (A_size < sides) {A_size = sides + 5; A = ALLOC(A_size,A,point);}
 
 	ND_label(n)->p = ND_coord_i(n);
-/* prescale by 16.0 to help rounding trick below */
-	xsize = (16.0 * (ND_lw_i(n) + ND_rw_i(n)) / POINTS(ND_width(n)));
-	ysize = (16.0 * ND_ht_i(n) / POINTS(ND_height(n)));
+	xsize = (double)(ND_lw_i(n) + ND_rw_i(n)) / POINTS(ND_width(n));
+	ysize = (double)(ND_ht_i(n)) / POINTS(ND_height(n));
 
 	/* this is bad, but it's because of how the VRML driver works */
 #ifdef HAVE_GD_PNG
@@ -859,11 +909,8 @@ static void poly_gencode(GVC_t *gvc, node_t *n)
 	if (ND_shape(n)->usershape) {
 		for (i = 0; i < sides; i++) {
 			P = vertices[i];
-/* simple rounding produces random results around .5 
- * this trick should clip off the random part. 
- * (note xsize/ysize prescaled by 16.0 above) */
-			A[i].x = ROUND(P.x * xsize) / 16;
-			A[i].y = ROUND(P.y * ysize) / 16;
+			A[i].x = ROUND(P.x * xsize);
+			A[i].y = ROUND(P.y * ysize);
 			if (sides > 2) {
 				A[i].x += ND_coord_i(n).x;
 				A[i].y += ND_coord_i(n).y;
@@ -883,11 +930,8 @@ static void poly_gencode(GVC_t *gvc, node_t *n)
 	for (j = 0; j < peripheries; j++) {
 		for (i = 0; i < sides; i++) {
 			P = vertices[i+j*sides];
-/* simple rounding produces random results around .5 
- * this trick should clip off the random part. 
- * (note xsize/ysize prescaled by 16.0 above) */
-			A[i].x = ROUND(P.x * xsize) / 16;
-			A[i].y = ROUND(P.y * ysize) / 16;
+			A[i].x = ROUND(P.x * xsize);
+			A[i].y = ROUND(P.y * ysize);
 			if (sides > 2) {
 				A[i].x += ND_coord_i(n).x;
 				A[i].y += ND_coord_i(n).y;
@@ -1290,59 +1334,23 @@ static field_t	* map_rec_port(field_t* f, char* str)
 	return rv;
 }
 
-int
-compassPort (node_t* n, box* bp, port* pp, char* terminal)
-{
-	box   b = *bp;
-	point p;
-	int   rv = 0;
-
-	p = pointof((b.LL.x+b.UR.x)/2,(b.LL.y+b.UR.y)/2);
-	if (terminal) {
-		int margin = 1;	/* would like = 0, but spline router croaks */
-		switch(*terminal) {
-		case 'e':	p.x = b.UR.x - margin; break;
-		case 's':	p.y = b.LL.y + margin; break;
-		case 'w':	p.x = b.LL.x + margin; break;
-		case 'n':	p.y = b.UR.y - margin; break;
-		default :   rv = 1;
-		}
-	}
-	p = invflip_pt(p, GD_rankdir(n->graph));
-	pp->bp = bp;
-	pp->p = p;
-	pp->order = (MC_SCALE * (ND_lw_i(n) + p.x)) / (ND_lw_i(n) + ND_rw_i(n));
-	pp->constrained = FALSE;
-	pp->defined = TRUE;
-	return rv;
-}
-
-static port record_port(node_t *n, char* portname)
+static port record_port(node_t *n, char* portname, char* compass)
 {
 	field_t	*f;
+	field_t	*subf;
 	port	rv;
-	char	*terminal;
 
-	if (portname[0] != ':') return Center;		/*could be '\000' */
-	portname++;   /* skip delimiter */
-
-	/* look for possible modifier port.{n,s,e,w} */
-	if ((terminal = strchr(portname,'.'))) {
-		char *buf = strdup(portname);
-		buf[terminal - portname] = 0;
-		portname = buf;		/* swap in properly terminated portname */
-		terminal++;
+	if (portname[0] == '\0') return Center;
+	f = (field_t*) ND_shape_info(n);
+	if ((subf = map_rec_port(f,portname))) {
+		if (compassPort (n, &subf->b, &rv, compass))
+			agerr(AGWARN, "node %s, port %s, unrecognized compass point '%s' - ignored\n",
+			n->name,portname,compass);
 	}
-	if ((f = map_rec_port((field_t*) ND_shape_info(n),portname)) == NULL) {
+	else if (compassPort (n, &f->b, &rv, portname)) {
 		unrecognized(n,portname);
-		if (terminal) free(portname);
-		return Center;
 	}
 
-	if (compassPort (n, &f->b, &rv, terminal))
-		agerr(AGWARN, "node %s, port %s, unrecognized compass point '%c' - ignored\n",
-			n->name,portname,*terminal);
-	if (terminal) free(portname);
 	return rv;
 }
 
