@@ -700,12 +700,39 @@ edgeError (edge_t *e, char* msg)
 		msg, e->tail->name, edgeop, e->head->name);
 }
 
+struct fontinfo {
+	double  fontsize;
+	char*   fontname;
+	char*   fontcolor;
+};
+
+static void
+initFontEdgeAttr (edge_t *e, struct fontinfo* fi)
+{
+	fi->fontsize = late_double(e,E_fontsize,DEFAULT_FONTSIZE,MIN_FONTSIZE);
+	fi->fontname = late_nnstring(e,E_fontname,DEFAULT_FONTNAME);
+	fi->fontcolor = late_nnstring(e,E_fontcolor,DEFAULT_COLOR);
+}
+
+static void
+initFontLabelEdgeAttr (edge_t *e, struct fontinfo* fi, struct fontinfo* lfi)
+{
+	if (!fi->fontname) initFontEdgeAttr (e, fi);
+	lfi->fontsize = late_double(e,E_labelfontsize,fi->fontsize,MIN_FONTSIZE);
+	lfi->fontname = late_nnstring(e,E_labelfontname,fi->fontname);
+	lfi->fontcolor = late_nnstring(e,E_labelfontcolor,fi->fontcolor);
+}
+
 /* return TRUE if edge has label */
 int common_init_edge(edge_t *e)
 {
 	char    *s;
 	int     html = 0, r = 0;
+	struct fontinfo fi;
+	struct fontinfo lfi;
 
+	fi.fontname = NULL;
+	lfi.fontname = NULL;
 	if (E_label && (s = agxget(e,E_label->index)) && (s[0])) {
 		r = 1;
 		html = aghtmlstr(s);
@@ -713,10 +740,9 @@ int common_init_edge(edge_t *e)
 			s = strdup(s);
 		else
 			s = strdup_and_subst_edge(s,e);
+		initFontEdgeAttr (e, &fi);
 		ED_label(e) = make_label(html, s,
-			late_double(e,E_fontsize,DEFAULT_FONTSIZE,MIN_FONTSIZE),
-			late_nnstring(e,E_fontname,DEFAULT_FONTNAME),
-			late_nnstring(e,E_fontcolor,DEFAULT_COLOR),e->tail->graph);
+			fi.fontsize, fi.fontname, fi.fontcolor, e->tail->graph);
 		if (html) {
 			if (make_html_label(ED_label(e), e))
 				edgeError(e, "label");
@@ -725,6 +751,7 @@ int common_init_edge(edge_t *e)
 		ED_label_ontop(e) = mapbool(late_string(e,E_label_float,"false"));
 	}
 
+	
     /* vladimir */
 	if (E_headlabel && (s = agxget(e,E_headlabel->index)) && (s[0])) {
 		html = aghtmlstr(s);
@@ -732,10 +759,9 @@ int common_init_edge(edge_t *e)
 			s = strdup(s);
 		else
 			s = strdup_and_subst_edge(s,e);
-		ED_head_label(e) = make_label(0, s,
-			late_double(e,E_labelfontsize,DEFAULT_LABEL_FONTSIZE,MIN_FONTSIZE),
-			late_nnstring(e,E_labelfontname,DEFAULT_FONTNAME),
-			late_nnstring(e,E_labelfontcolor,DEFAULT_COLOR),e->tail->graph);
+		initFontLabelEdgeAttr (e, &fi, &lfi);
+		ED_head_label(e) = make_label(html, s,
+			lfi.fontsize, lfi.fontname, lfi.fontcolor, e->tail->graph);
 		if (html) {
 			if (make_html_label(ED_head_label(e), e))
 				edgeError(e, "head label");
@@ -748,10 +774,9 @@ int common_init_edge(edge_t *e)
 			s = strdup(s);
 		else
 			s = strdup_and_subst_edge(s,e);
-		ED_tail_label(e) = make_label(0, s,
-			late_double(e,E_labelfontsize,DEFAULT_LABEL_FONTSIZE,MIN_FONTSIZE),
-			late_nnstring(e,E_labelfontname,DEFAULT_FONTNAME),
-			late_nnstring(e,E_labelfontcolor,DEFAULT_COLOR),e->tail->graph);
+		if (!lfi.fontname) initFontLabelEdgeAttr (e, &fi, &lfi);
+		ED_tail_label(e) = make_label(html, s,
+			lfi.fontsize, lfi.fontname, lfi.fontcolor, e->tail->graph);
 		if (html) {
 			if (make_html_label(ED_tail_label(e), e))
 				edgeError(e, "tail label");
@@ -760,7 +785,17 @@ int common_init_edge(edge_t *e)
 	}
     /* end vladimir */
 
-        return r;
+	/* We still accept ports beginning with colons but this is deprecated */
+	s = agget(e,TAIL_ID);
+	if (s[0]) ND_has_port(e->tail) = TRUE;
+	if (s[0] == ':') s++;
+	ED_tail_port(e) = ND_shape(e->tail)->fns->portfn(e->tail,s,agget(e,TAILSUB_ID));
+	s = agget(e,HEAD_ID);
+	if (s[0]) ND_has_port(e->head) = TRUE;
+	if (s[0] == ':') s++;
+	ED_head_port(e) = ND_shape(e->head)->fns->portfn(e->head,s,agget(e,HEADSUB_ID));
+
+	return r;
 }
 
 point 
@@ -998,7 +1033,8 @@ clustNode (node_t* n, graph_t* cg, agxbuf* xb, graph_t* clg)
 typedef struct {
     Dtlink_t     link;     /* cdt data */
     void*        p[2];     /* key */
-    edge_t*      ep;       /* edge */
+    node_t*      t;       
+    node_t*      h;      
 } item;
 
 static int
@@ -1024,7 +1060,8 @@ newItem (Dt_t* d, item* objp, Dtdisc_t* disc)
     NOTUSED(disc);
     newp->p[0] = objp->p[0];
     newp->p[1] = objp->p[1];
-    newp->ep = objp->ep;
+    newp->t = objp->t;
+    newp->h = objp->h;
 
     return newp;
 }
@@ -1050,17 +1087,17 @@ static Dtdisc_t mapDisc = {
 } ;
 
 /* cloneEdge:
- * Make a copy of e in e's graph but using t and h as endnodes
+ * Make a copy of e in e's graph but using ct and ch as nodes
  */
 static edge_t*
-cloneEdge (edge_t* e, node_t* t, node_t* h)
+cloneEdge (edge_t* e, node_t* ct, node_t* ch)
 {
   extern Agdict_t*  agdictof(void*);
   Agdict_t* d = agdictof(e);
   Agsym_t** list = d->list;
   Agsym_t*  sym;
-  graph_t*  g = t->graph;
-  edge_t*   ce = agedge (g, t, h);
+  graph_t*  g = ct->graph;
+  edge_t*   ce = agedge (g, ct, ch);
 
   while ((sym = *list++)) {
     agxset (ce, sym->index, agxget(e,sym->index));
@@ -1070,51 +1107,52 @@ cloneEdge (edge_t* e, node_t* t, node_t* h)
 }
 
 /* insertEdge:
- * Make a copy of e in e's graph but using t and h as endnodes
  */
 static void
 insertEdge (Dt_t* map, void* t, void* h, edge_t* e)
 {
   item    dummy;
 
-  if (t < h) {
-    dummy.p[0] = t;
-    dummy.p[1] = h;
-  }
-  else {
-    dummy.p[0] = h;
-    dummy.p[1] = t;
-  }
-  dummy.ep = e;
+  dummy.p[0] = t;
+  dummy.p[1] = h;
+  dummy.t = e->tail;
+  dummy.h = e->head;
+  dtinsert (map, &dummy);
+  
+  dummy.p[0] = h;
+  dummy.p[1] = t;
+  dummy.t = e->head;
+  dummy.h = e->tail;
   dtinsert (map, &dummy);
 }
 
 /* mapEdge:
- * Make a copy of e in e's graph but using t and h as endnodes
+ * Check if we already have cluster edge corresponding to t->h,
+ * and return it.
  */
-static edge_t*
-mapEdge (Dt_t* map, void* t, void* h)
+static item*
+mapEdge (Dt_t* map, edge_t* e)
 {
-  item*   ip;
   void*   key[2];
 
-  if (t < h) {
-    key[0] = t;
-    key[1] = h;
-  }
-  else {
-    key[0] = h;
-    key[1] = t;
-  }
-  ip = dtmatch (map, &key);
-  if (ip) return ip->ep;
-  else return NULL;
+  key[0] = e->tail;
+  key[1] = e->head;
+  return (item*)dtmatch (map, &key);
 }
 
 /* checkCompound:
  * If endpoint names a cluster, mark for temporary deletion and create 
  * special node and insert into cluster. Then clone the edge. Real edge
  * will be deleted when we delete the original node.
+ * Invariant: new edge has same sense as old. That is, given t->h with
+ * t and h mapped to ct and ch, the new edge is ct->ch.
+ *
+ * In the current model, we create a cluster node for each cluster edge
+ * between the cluster and some other node or cluster, treating the
+ * cluster node as a port on the cluster. This should help with better
+ * routing to avoid edge crossings. At present, this is not implemented,
+ * so we could use a simpler model in which we create a single cluster
+ * node for each cluster used in a cluster edge.
  */
 #define MAPC(n) (strncmp((n)->name,"cluster",7)?NULL:agfindsubg((n)->graph, (n)->name))
 
@@ -1128,6 +1166,7 @@ checkCompound (edge_t* e, graph_t* clg, agxbuf* xb, Dt_t* map)
   node_t*  t = e->tail;
   node_t*  h = e->head;
   edge_t*  ce;
+  item*    ip;
 
   tg = MAPC(t);
   hg = MAPC(h);
@@ -1136,9 +1175,9 @@ checkCompound (edge_t* e, graph_t* clg, agxbuf* xb, Dt_t* map)
     agerr(AGWARN, "cluster cycle %s -- %s not supported\n", t->name,t->name);
     return;
   }
-  ce = mapEdge (map, t, h);
-  if (ce) {
-    cloneEdge (e, ce->tail, ce->head);
+  ip = mapEdge (map, e);
+  if (ip) {
+    cloneEdge (e, ip->t, ip->h);
     return;
   }
 
@@ -1182,9 +1221,16 @@ checkCompound (edge_t* e, graph_t* clg, agxbuf* xb, Dt_t* map)
   }
 }
 
-void
+/* processClusterEdges:
+ * Look for cluster edges. Replace cluster edge endpoints
+ * corresponding to a cluster with special cluster nodes.
+ * Delete original nodes.
+ * Return 0 if no cluster edges; 1 otherwise.
+ */
+int
 processClusterEdges (graph_t *g)
 {
+  int     rv;
   node_t* n;
   edge_t* e;
   graph_t *clg;
@@ -1202,9 +1248,90 @@ processClusterEdges (graph_t *g)
   }
   agxbfree (&xb);
   dtclose (map);
+  rv = agnnodes(clg);
   for (n = agfstnode(clg); n; n = agnxtnode(clg,n)) {
     agdelete (g, n);
   }
   agclose (clg);
-
+  if (rv) SET_CLUST_EDGE(g);
+  return rv;
 }
+
+/* mapN:
+ * Convert cluster nodes back to ordinary nodes
+ * If n is already ordinary, return it.
+ * Otherwise, we know node's name is "__i:xxx"
+ * where i is some number and xxx is the nodes's original name.
+ * Create new node of name xxx if it doesn't exist and add n to clg
+ * for later deletion.
+ */ 
+static node_t*
+mapN (node_t* n, graph_t* clg)
+{
+  extern Agdict_t*  agdictof(void*);
+  node_t*  nn;
+  char*    name;
+  graph_t* g = n->graph;
+  Agdict_t* d;
+  Agsym_t** list;
+  Agsym_t*  sym;
+
+  if(!(IS_CLUST_NODE(n))) return n;
+  aginsert (clg, n);
+
+  name = strchr (n->name, ':');
+  assert (name);
+  name++;
+  if ((nn = agfindnode(g, name))) return nn;
+  nn = agnode(g, name);
+
+    /* Set all attributes to default */
+  d = agdictof(n);
+  list = d->list;
+  while ((sym = *list++)) {
+      /* Can use pointer comparison because of ref strings. */ 
+    if (agxget(nn,sym->index) != sym->value)  
+      agxset (nn, sym->index, sym->value);    
+  }
+
+  return nn;
+}
+
+static void
+undoCompound (edge_t* e, graph_t* clg)
+{
+  node_t*  t = e->tail;
+  node_t*  h = e->head;
+  node_t*  ntail;
+  node_t*  nhead;
+
+  if(!(IS_CLUST_NODE(t) || IS_CLUST_NODE(h))) return; 
+  ntail = mapN (t, clg);
+  nhead = mapN (h, clg);
+  cloneEdge (e, ntail, nhead);
+}
+
+/* undoClusterEdges:
+ * Replace cluster nodes with originals. Make sure original has
+ * no attributes. Replace original edges. Delete cluster nodes,
+ * which will also delete cluster edges.
+ */
+void
+undoClusterEdges (graph_t *g)
+{
+  node_t* n;
+  edge_t* e;
+  graph_t *clg;
+
+  clg = agsubg (g, "__clusternodes");
+  for (n = agfstnode(g); n; n = agnxtnode(g,n)) {
+    for (e = agfstout(g,n); e; e = agnxtout(g,e)) {
+      undoCompound (e,clg);
+    }
+  }
+  for (n = agfstnode(clg); n; n = agnxtnode(clg,n)) {
+    agdelete (g, n);
+  }
+  agclose (clg);
+}
+
