@@ -7,7 +7,6 @@
     with AT&T, you have an infringing copy of this software and cannot use
     it without violating AT&T's intellectual property rights.
 */
-#pragma prototyped
 
 /*
  * Break cycles in a directed graph by depth-first search.
@@ -15,9 +14,6 @@
 
 #include "dot.h"
 
-#ifdef DMALLOC
-#include "dmalloc.h"
-#endif
 
 void acyclic(graph_t* g)
 {
