@@ -7,7 +7,6 @@
     with AT&T, you have an infringing copy of this software and cannot use
     it without violating AT&T's intellectual property rights.
 */
-#pragma prototyped
 
 /*
  * Classify edges for rank assignment phase to
@@ -16,9 +15,6 @@
 
 #include "dot.h"
 
-#ifdef DMALLOC
-#include "dmalloc.h"
-#endif
 
 int nonconstraint_edge(edge_t* e)
 {
