@@ -10,9 +10,6 @@
 
 #ifndef		DOT_H
 #define		DOT_H
-#ifdef _PACKAGE_AST
-#include <ast.h>
-#endif
 
 #include	"render.h"
 #include	"dotprocs.h"
