@@ -1722,7 +1722,7 @@ static void beginpath (edge_t* e, int et, pathend_t* endp)
 		/* moving the box UR.y by + 1 avoids colinearity between
 			port point and box that confuses Proutespline().  it's
 			a bug in Proutespline() but this is the easiest fix. */
-		endp->boxes[0].UR.y = P->start.p.y + 1;
+		endp->boxes[0].UR.y = P->start.p.y - 1;
 		endp->sidemask = BOTTOM;
 		break;
 	case FLATEDGE:
@@ -1773,7 +1773,7 @@ static void endpath (edge_t* e, int et, pathend_t* endp)
 		/* offset of -1 is symmetric w.r.t. beginpath() 
 		 * FIXME: is any of this right?  what if self-edge 
 		 * doesn't connect from BOTTOM to TOP??? */
-		endp->boxes[0].LL.y = P->end.p.y - 1;
+		endp->boxes[0].LL.y = P->end.p.y + 1;
 		endp->sidemask = TOP;
 		break;
 	case FLATEDGE:
