@@ -7,7 +7,6 @@
     with AT&T, you have an infringing copy of this software and cannot use
     it without violating AT&T's intellectual property rights.
 */
-#pragma prototyped
 
 /*
  * Rank the nodes of a directed graph, subject to user-defined
@@ -25,9 +24,6 @@
 
 #include	"dot.h"
 
-#ifdef DMALLOC
-#include "dmalloc.h"
-#endif
 
 void dot_rank(graph_t* g)
 {
@@ -52,7 +48,7 @@ void edgelabel_ranks(graph_t* g)
 	node_t		*n;
 	edge_t		*e;
 
-	if (GD_has_edge_labels(g)) {
+	if (GD_has_labels(g)&EDGE_LABEL) {
 		for (n = agfstnode(g); n; n = agnxtnode(g,n))
 			for (e = agfstout(g,n); e; e = agnxtout(g,e))
 				ED_minlen(e) *= 2;
@@ -153,9 +149,9 @@ node_t* merge_leaves(graph_t *g, node_t *cur, node_t *new)
 	if (cur == NULL) rv = new;
 	else {
 		rv = UF_union(cur,new);
-		ND_ht(rv) = MAX(ND_ht(cur),ND_ht(new));
-		ND_lw(rv) = ND_lw(cur) + ND_lw(new) + GD_nodesep(g)/2;
-		ND_rw(rv) = ND_rw(cur) + ND_rw(new) + GD_nodesep(g)/2;
+		ND_ht_i(rv) = MAX(ND_ht_i(cur),ND_ht_i(new));
+		ND_lw_i(rv) = ND_lw_i(cur) + ND_lw_i(new) + GD_nodesep(g)/2;
+		ND_rw_i(rv) = ND_rw_i(cur) + ND_rw_i(new) + GD_nodesep(g)/2;
 	}
 	return rv;
 }
@@ -362,9 +358,9 @@ void renewlist(elist* L)
 
 void cleanup1(graph_t* g)
 {
-	node_t		*n;
-	edge_t		*e, *f;
-	int			c;
+	node_t	*n;
+	edge_t	*e, *f;
+	int		c;
 
 	for (c = 0; c < GD_comp(g).size; c++) {
 		GD_nlist(g) = GD_comp(g).list[c];
@@ -377,7 +373,21 @@ void cleanup1(graph_t* g)
 	for (n = agfstnode(g); n; n = agnxtnode(g,n)) {
 		for (e = agfstout(g,n); e; e = agnxtout(g,e)) {
 			f = ED_to_virt(e);
-			if (f && (e == ED_to_orig(f))) free(f);
+			if (f && (e == ED_to_orig(f))) {
+	/* Null out any other references to f to make sure we don't handle it
+	 * a second time. For example, parallel multiedges share a virtual edge.
+	 */
+				edge_t	*e1, *f1;
+				for (e1 = agfstout(g,n); e1; e1 = agnxtout(g,e1)) {
+					if (e != e1) {
+						f1 = ED_to_virt(e1);
+						if (f1 && (f == f1)) {
+							ED_to_virt(e1) = NULL;
+						}
+					}
+				}
+				free(f);
+			}
 			ED_to_virt(e) = NULL;
 		}
 	}
