@@ -714,6 +714,7 @@ setClustNodes(graph_t* g)
   bport_t*    pp;
   node_t*     n;
   double      w, h;
+  int         h2, w2, h_i;
   pointf*     vertices;
   
   pp = PORTS(g);
@@ -722,6 +723,9 @@ setClustNodes(graph_t* g)
   ctr.y =  (bb.UR.y + bb.LL.y)/2.0;
   w =  bb.UR.x - bb.LL.x;
   h =  bb.UR.y - bb.LL.y;
+  w2 = POINTS(w/2.0);
+  h2 = POINTS(h/2.0);
+  h_i = POINTS(h);
   while (pp->e) {
     if (IS_CLUST_NODE(pp->n) && (PARENT(pp->n) == g)) {
       n = pp->n;
@@ -730,19 +734,18 @@ setClustNodes(graph_t* g)
       ND_width(n) = w;
       ND_height(n) = h;
       ND_xsize(n) = POINTS(w);
-      ND_lw_i(n)  = ND_rw_i(n) = POINTS(w/2.0);
-      h = POINTS(h);
-      ND_ht_i(n) = ND_ysize(n) = h;
+      ND_lw_i(n)  = ND_rw_i(n) = w2;
+      ND_ht_i(n) = ND_ysize(n) = h_i;
 
       vertices = ((polygon_t*) ND_shape_info(n))->vertices;
       vertices[0].x = ND_rw_i(n);
-      vertices[0].y = h/2.0;
+      vertices[0].y = h2;
       vertices[1].x = -ND_lw_i(n);
-      vertices[1].y = h/2.0;
+      vertices[1].y = h2;
       vertices[2].x = -ND_lw_i(n);
-      vertices[2].y = -h/2.0;
+      vertices[2].y = -h2;
       vertices[3].x = ND_rw_i(n);
-      vertices[3].y = -h/2.0;
+      vertices[3].y = -h2;
     }
     pp++;
   }
@@ -945,9 +948,21 @@ fdpLayout (graph_t* g)
 void
 fdp_layout (graph_t* g)
 {
+  char*    str = NULL;
+
   fdp_init_graph (g);
   fdpLayout (g);
+  if (HAS_CLUST_EDGE(g)) {
+    str = agget(g,"splines");
+    if (mapbool(str)) {
+      agerr(AGWARN,"splines and cluster edges not supported - using line segments\n");
+      agset(g,"splines","false");
+    }
+    else str = NULL;
+  }
   spline_edges0(g);
+  if (str)
+    agset(g,"splines",str);
   dotneato_postprocess(g, neato_nodesize);
 }
 
