@@ -44,6 +44,13 @@
 #include <dbg.h>
 #include <grid.h>
 
+#ifndef HAVE_SRAND48
+#define srand48 srand
+#endif
+#ifndef HAVE_DRAND48
+extern double drand48(void);
+#endif
+
   /* Default layout values; -1 indicates unset */
 tParms_t fdp_tvals = {
   1,     /* useGrid */
@@ -360,6 +367,8 @@ updatePos (Agraph_t* g, double temp, bport_t* pp)
   }
 }
 
+#define FLOOR(d) ((int)floor(d))
+
 /* gAdjust:
  */
 static void
@@ -374,7 +383,7 @@ gAdjust (Agraph_t* g, double temp, bport_t* pp, Grid* grid)
 
   for (n = agfstnode(g); n; n = agnxtnode(g,n)) {
     DISP(n)[0] = DISP(n)[1] = 0;
-    addGrid (grid, floor(n->u.pos[0]/Cell), floor(n->u.pos[1]/Cell), n);
+    addGrid (grid, FLOOR(n->u.pos[0]/Cell), FLOOR(n->u.pos[1]/Cell), n);
   }
 
   for (n = agfstnode(g); n; n = agnxtnode(g,n)) {
