@@ -7,7 +7,6 @@
     with AT&T, you have an infringing copy of this software and cannot use
     it without violating AT&T's intellectual property rights.
 */
-#pragma prototyped
 /* adjust.c
  * Routines for repositioning nodes after initial layout in
  * order to reduce/remove node overlaps.
@@ -19,11 +18,8 @@
 #include "info.h"
 #include "edges.h"
 #include "site.h"
-#include "info.h"
-
-#ifdef DMALLOC
-#include "dmalloc.h"
-#endif
+#include "heap.h"
+#include "hedges.h"
 
 static double    margin = 0.05;     /* Create initial bounding box by adding
                                     * margin * dimension around box enclosing
@@ -32,7 +28,7 @@ static double    margin = 0.05;     /* Create initial bounding box by adding
 static double    incr = 0.05;       /* Increase bounding box by adding
                                     * incr * dimension around box.
                                     */
-static double    pmargin = 5.0/72;  /* Margin around polygons, in inches */
+static double    pmargin = 5.0/POINTS_PER_INCH;  /* Margin around polygons, in inches */
 static int      iterations = -1;  /* Number of iterations */
 static int      useIter    = 0;   /* Use specified number of iterations */
 
@@ -147,7 +143,7 @@ makeInfo(Agraph_t* graph)
     nsites = agnnodes (graph);
     geominit ();
 
-    nodeInfo = (Info_t *) malloc(nsites * (sizeof (Info_t)));
+    nodeInfo = N_GNEW(nsites,Info_t);
 
     node = agfstnode (graph);
     ip = nodeInfo;
@@ -203,7 +199,7 @@ sortSites ()
     Info_t*      ip;
 
     if (sites == 0) {
-        sites = (Site**)malloc(nsites * sizeof(Site*));
+        sites = N_GNEW(nsites, Site*);
         endSite = sites + nsites;
     }
 
@@ -225,11 +221,11 @@ sortSites ()
 }
 
 static void
-geomUpdate ()
+geomUpdate (int doSort)
 {
     int      i;
 
-    sortSites ();
+    if (doSort) sortSites ();
 
     /* compute ranges */
     xmin=sites[0]->coord.x; 
@@ -258,6 +254,68 @@ Site *nextOne()
         return((Site *)NULL);
 }
 
+/* rmEquality:
+ * Check for nodes with identical positions and tweak
+ * the positions.
+ */
+static void
+rmEquality ()
+{
+    int         i, cnt;
+    Site**      ip;
+    Site**      jp;
+    Site**      kp;
+    double      xdel;
+
+    sortSites ();
+    ip = sites;
+    
+    while (ip < endSite) {
+      jp = ip+1;
+      if ((jp >= endSite) || 
+          ((*jp)->coord.x != (*ip)->coord.x) ||
+          ((*jp)->coord.y != (*ip)->coord.y)) {
+        ip = jp;
+        continue;
+      }
+
+        /* Find first node kp with position different from ip */
+      cnt = 2;
+      kp = jp+1;
+      while ((kp < endSite) &&
+          ((*kp)->coord.x == (*ip)->coord.x) &&
+          ((*kp)->coord.y == (*ip)->coord.y)) {
+        cnt++;
+        jp = kp;
+        kp = jp+1;
+      }
+
+        /* If next node exists and is on the same line */
+      if ((kp < endSite) && ((*kp)->coord.y == (*ip)->coord.y)) {
+        xdel = ((*kp)->coord.x - (*ip)->coord.x)/cnt;
+        i = 1;
+        for (jp = ip+1; jp < kp; jp++) {
+          (*jp)->coord.x += i*xdel;
+          i++;
+        }
+      }
+      else { /* nothing is to the right */
+        Info_t*      info;
+        for (jp = ip+1; jp < kp; ip++,jp++) {
+          info = nodeInfo + (*ip)->sitenbr;
+          xdel = info->poly.corner.x - info->poly.origin.x;
+          info = nodeInfo + (*jp)->sitenbr;
+          xdel += info->poly.corner.x - info->poly.origin.x;
+          (*jp)->coord.x = (*ip)->coord.x + xdel/2;
+        }
+      }
+      ip = kp;
+    }
+}
+
+/* countOverlap:
+ * Count number of node-node overlaps at iteration iter.
+ */
 static int
 countOverlap (int iter)
 {
@@ -437,6 +495,23 @@ newPos ()
     }
 }
 
+/* cleanup:
+ * Cleanup voronoi memory.
+ * Note that PQcleanup and ELcleanup rely on the number
+ * of sites, so should at least be reset everytime we use
+ * vAdjust.
+ * This could be optimized, over multiple components or
+ * even multiple graphs, but probably not worth it.
+ */
+static void
+cleanup ()
+{
+	PQcleanup();
+	ELcleanup();
+	siteinit();  /* free memory */
+	edgeinit();  /* free memory */
+}
+
 static int
 vAdjust ()
 {
@@ -452,7 +527,8 @@ vAdjust ()
     if ((overlapCnt == 0) || (iterations == 0))
       return 0;
 
-    geomUpdate ();
+    rmEquality ();
+    geomUpdate (0);
     voronoi(0, nextOne); 
     while (1) {
       newPos ();
@@ -481,7 +557,7 @@ vAdjust ()
         break;
       }
 
-      geomUpdate ();
+      geomUpdate (1);
       voronoi(0, nextOne); 
     }
 
@@ -490,6 +566,7 @@ vAdjust ()
       fprintf (stderr, "Number of increases = %d\n", increaseCnt);
     }
 
+	cleanup ();
     return 1;
 }
 
@@ -524,6 +601,7 @@ sAdjust ()
     if ((overlapCnt == 0) || (iterations == 0))
       return 0;
 
+    rmEquality ();
     center.x =  (pxmin + pxmax)/2.0;
     center.y =  (pymin + pymax)/2.0;
     while (1) {
@@ -598,6 +676,7 @@ void adjustNodes (graph_t* G)
     int          doScale = 0;
     int          ret;
 
+	if (agnnodes(G) < 2) return;
     normalize(G);
     flag = agget(G,"overlap");
     if (flag == NULL) return;
@@ -619,5 +698,5 @@ void adjustNodes (graph_t* G)
 
     freeNodes ();
     free (sites);
-
+	sites = 0;
 }
