@@ -7,13 +7,9 @@
     with AT&T, you have an infringing copy of this software and cannot use
     it without violating AT&T's intellectual property rights.
 */
-#pragma prototyped
 #include "geometry.h"
 #include <math.h>
 
-#ifdef DMALLOC
-#include "dmalloc.h"
-#endif
 
 Point origin = {0,0};
 
