@@ -7,13 +7,9 @@
     with AT&T, you have an infringing copy of this software and cannot use
     it without violating AT&T's intellectual property rights.
 */
-#pragma prototyped
 #include "neato.h"
 #include "simple.h"
 
-#ifdef DMALLOC
-#include "dmalloc.h"
-#endif
 
 static void
 sgnarea(l,m,i) struct vertex *l,*m; int i[];
@@ -117,7 +113,7 @@ find_intersection(struct vertex *l,
 			return;
 
 	if (input->ninters >= MAXINTS) 	{
-		fprintf(stderr,"\n**ERROR**\n using too many intersections\n");
+		agerr(AGERR, "using too many intersections\n");
 		exit(1);		}
 
 	ilist[input->ninters].firstv  = l;
