@@ -21,7 +21,12 @@
  *	n    - the order of the matrices A and Ainv
  */
 
+#if _PACKAGE_ast
+#include <ast.h>
+#else
+#endif
 #include <stdlib.h>
+#include <render.h>
 extern int lu_decompose(double **a, int n);
 extern void lu_solve(double *x, double *b, int n);
 
@@ -35,7 +40,7 @@ int matinv(double **A, double **Ainv, int n)
 	     return(0);		/* Singular */
 
 	/* Invert matrix by solving n simultaneous equations n times */
-	b = calloc(n,sizeof(double));
+	b = N_NEW(n,double);
 	for (i = 0; i < n; i++) {
 	    for(j = 0; j < n; j++)
 		b[j] = 0.0;
