@@ -10,16 +10,24 @@
 
 #ifndef         NEATO_H
 #define         NEATO_H
-#ifdef _PACKAGE_AST
-#include <ast.h>
-#endif
 
 #ifdef HAVE_CONFIG_H
 #include "config.h"
 #endif
 
-#include	"pathplan.h"
+#define MODEL_SHORTPATH      0
+#define MODEL_CIRCUIT        1
+#define MODEL_SUBSET         2
+
+#define MODE_KK          0
+#define MODE_MAJOR       1
+
+#define INIT_SELF        0
+#define INIT_REGULAR     1
+#define INIT_RANDOM      2
+
 #include	"render.h"
+#include	"pathplan.h"
 #include	"neatoprocs.h"
 #include	"adjust.h"
 
