@@ -54,9 +54,6 @@ void neato_init_edge(edge_t* e)
 	common_init_edge(e);
 
 	ED_factor(e) = late_double(e,E_weight,1.0,1.0);
-
-	init_port(e->tail,e,agget(e,"tailport"), FALSE);
-	init_port(e->head,e,agget(e,"headport"), TRUE);
 }
 
 int 
@@ -106,23 +103,6 @@ void neato_init_node_edge(graph_t *g)
     }
 }
 
-int init_port(node_t* n, edge_t* e, char* name, boolean isHead)
-{
-	port	p0;
-
-	if (!name[0])
-		return FALSE;
-	ND_has_port(n) = TRUE;
-	p0 = ND_shape(n)->fns->portfn(n,name);
-#ifdef NOTDEF
-	p0.p = invflip_pt(p0.p, n->GD_rankdir(graph));
-#endif 
-	p0.order = 0;
-	if (isHead) ED_head_port(e) = p0;
-	else ED_tail_port(e) = p0;
-	return TRUE;
-}
-
 void neato_cleanup_node(node_t* n)
 {
 	if (ND_shape(n)) {
@@ -522,7 +502,7 @@ init_nop (Agraph_t *g)
 
    	scan_graph(g);       /* mainly to set up GD_neato_nlist */
 	for (i = 0; (np = GD_neato_nlist(g)[i]); i++) {
-		if (!hasPos(np)) {
+		if (!hasPos(np) && strncmp(np->name,"cluster",7)) {
 			agerr(AGERR, "node %s in graph %s has no position\n",
 				np->name, g->name);
 			return 1;
