@@ -16,6 +16,9 @@
  * Modification 4/18/00 TBB: JPEG_DEBUG rather than just DEBUG,
  * so VC++ builds don't spew to standard output, causing
  * major CGI brain damage
+ *
+ * 2.0.10: more efficient gdImageCreateFromJpegCtx, thanks to
+ * Christian Aberger 
  */
 
 #ifdef HAVE_CONFIG_H
@@ -34,6 +37,12 @@
 #ifdef HAVE_LIBJPEG
 #include "gdhelpers.h"
 
+/* JCE undef two symbols that we don't need anymore but which are
+   may be defined in config.h from ./configure but which are
+   redefined incompatibly in jpeglib.h */
+#undef HAVE_STDDEF_H
+#undef HAVE_STDLIB_H
+
 /* 1.8.1: remove dependency on jinclude.h */
 #include "jpeglib.h"
 #include "jerror.h"
@@ -41,9 +50,9 @@
 static const char *const GD_JPEG_VERSION = "1.0";
 
 typedef struct _jmpbuf_wrapper
-  {
-    jmp_buf jmpbuf;
-  }
+{
+  jmp_buf jmpbuf;
+}
 jmpbuf_wrapper;
 
 /* Called by the IJG JPEG library upon encountering a fatal error */
@@ -83,16 +92,14 @@ fatal_jpeg_error (j_common_ptr cinfo)
  * should be near optimal for many applications).  See the IJG JPEG
  * library documentation for more details.  */
 
-void
-gdImageJpeg (gdImagePtr im, FILE * outFile, int quality)
+BGD_DECLARE(void) gdImageJpeg (gdImagePtr im, FILE * outFile, int quality)
 {
   gdIOCtx *out = gdNewFileCtx (outFile);
   gdImageJpegCtx (im, out, quality);
   out->gd_free (out);
 }
 
-void *
-gdImageJpegPtr (gdImagePtr im, int *size, int quality)
+BGD_DECLARE(void *) gdImageJpegPtr (gdImagePtr im, int *size, int quality)
 {
   void *rv;
   gdIOCtx *out = gdNewDynamicCtx (2048, NULL);
@@ -104,8 +111,7 @@ gdImageJpegPtr (gdImagePtr im, int *size, int quality)
 
 void jpeg_gdIOCtx_dest (j_compress_ptr cinfo, gdIOCtx * outfile);
 
-void
-gdImageJpegCtx (gdImagePtr im, gdIOCtx * outfile, int quality)
+BGD_DECLARE(void) gdImageJpegCtx (gdImagePtr im, gdIOCtx * outfile, int quality)
 {
   struct jpeg_compress_struct cinfo;
   struct jpeg_error_mgr jerr;
@@ -186,8 +192,7 @@ gdImageJpegCtx (gdImagePtr im, gdIOCtx * outfile, int quality)
   sprintf (comment, "CREATOR: gd-jpeg v%s (using IJG JPEG v%d),",
 	   GD_JPEG_VERSION, JPEG_LIB_VERSION);
   if (quality >= 0)
-    sprintf (comment + strlen (comment), " quality = %d\n",
-	     quality);
+    sprintf (comment + strlen (comment), " quality = %d\n", quality);
   else
     strcat (comment + strlen (comment), " default quality\n");
   jpeg_write_marker (&cinfo, JPEG_COM, (unsigned char *) comment,
@@ -195,11 +200,12 @@ gdImageJpegCtx (gdImagePtr im, gdIOCtx * outfile, int quality)
   if (im->trueColor)
     {
 #if BITS_IN_JSAMPLE == 12
-      fprintf (stderr, "gd-jpeg: error: jpeg library was compiled for 12-bit\n"
-	 "precision. This is mostly useless, because JPEGs on the web are\n"
-	 "8-bit and such versions of the jpeg library won't read or write\n"
+      fprintf (stderr,
+	       "gd-jpeg: error: jpeg library was compiled for 12-bit\n"
+	       "precision. This is mostly useless, because JPEGs on the web are\n"
+	       "8-bit and such versions of the jpeg library won't read or write\n"
 	       "them. GD doesn't support these unusual images. Edit your\n"
-	 "jmorecfg.h file to specify the correct precision and completely\n"
+	       "jmorecfg.h file to specify the correct precision and completely\n"
 	       "'make clean' and 'make install' libjpeg again. Sorry.\n");
       goto error;
 #endif /* BITS_IN_JSAMPLE == 12 */
@@ -256,8 +262,7 @@ gdImageJpegCtx (gdImagePtr im, gdIOCtx * outfile, int quality)
   gdFree (row);
 }
 
-gdImagePtr
-gdImageCreateFromJpeg (FILE * inFile)
+BGD_DECLARE(gdImagePtr) gdImageCreateFromJpeg (FILE * inFile)
 {
   gdImagePtr im;
   gdIOCtx *in = gdNewFileCtx (inFile);
@@ -266,16 +271,24 @@ gdImageCreateFromJpeg (FILE * inFile)
   return im;
 }
 
-void
-  jpeg_gdIOCtx_src (j_decompress_ptr cinfo,
-		    gdIOCtx * infile);
+BGD_DECLARE(gdImagePtr) gdImageCreateFromJpegPtr (int size, void *data)
+{
+  gdImagePtr im;
+  gdIOCtx *in = gdNewDynamicCtxEx (size, data, 0);
+  im = gdImageCreateFromJpegCtx (in);
+  in->gd_free (in);
+  return im;
+}
+
+void jpeg_gdIOCtx_src (j_decompress_ptr cinfo, gdIOCtx * infile);
+
+static int CMYKToRGB (int c, int m, int y, int k, int inverted);
 
 /* 
  * Create a gd-format image from the JPEG-format INFILE.  Returns the
  * image, or NULL upon error.
  */
-gdImagePtr
-gdImageCreateFromJpegCtx (gdIOCtx * infile)
+BGD_DECLARE(gdImagePtr) gdImageCreateFromJpegCtx (gdIOCtx * infile)
 {
   struct jpeg_decompress_struct cinfo;
   struct jpeg_error_mgr jerr;
@@ -286,7 +299,8 @@ gdImageCreateFromJpegCtx (gdIOCtx * infile)
   JSAMPROW rowptr[1];
   int i, j, retval;
   JDIMENSION nrows;
-
+  int channels = 3;
+  int inverted = 0;
 #ifdef JPEG_DEBUG
   printf ("gd-jpeg: gd JPEG version %s\n", GD_JPEG_VERSION);
   printf ("gd-jpeg: JPEG library version %d, %d-bit sample values\n",
@@ -314,6 +328,10 @@ gdImageCreateFromJpegCtx (gdIOCtx * infile)
 
   jpeg_gdIOCtx_src (&cinfo, infile);
 
+  /* 2.0.22: save the APP14 marker to check for Adobe Photoshop CMYK
+     files with inverted components. */
+  jpeg_save_markers (&cinfo, JPEG_APP0 + 14, 256);
+
   retval = jpeg_read_header (&cinfo, TRUE);
   if (retval != JPEG_HEADER_OK)
     fprintf (stderr, "gd-jpeg: warning: jpeg_read_header returns"
@@ -322,8 +340,7 @@ gdImageCreateFromJpegCtx (gdIOCtx * infile)
   if (cinfo.image_height > INT_MAX)
     fprintf (stderr, "gd-jpeg: warning: JPEG image height (%u) is"
 	     " greater than INT_MAX (%d) (and thus greater than"
-	     " gd can handle)", cinfo.image_height,
-	     INT_MAX);
+	     " gd can handle)", cinfo.image_height, INT_MAX);
 
   if (cinfo.image_width > INT_MAX)
     fprintf (stderr, "gd-jpeg: warning: JPEG image width (%u) is"
@@ -334,16 +351,21 @@ gdImageCreateFromJpegCtx (gdIOCtx * infile)
 			       (int) cinfo.image_height);
   if (im == 0)
     {
-      fprintf (stderr, "gd-jpeg error: cannot allocate gdImage"
-	       " struct\n");
+      fprintf (stderr, "gd-jpeg error: cannot allocate gdImage" " struct\n");
       goto error;
     }
-
-  /*
-   * Force the image into RGB colorspace, but don't 
-   * reduce the number of colors anymore (GD 2.0) 
-   */
-  cinfo.out_color_space = JCS_RGB;
+  /* 2.0.22: very basic support for reading CMYK colorspace files. Nice for
+     thumbnails but there's no support for fussy adjustment of the
+     assumed properties of inks and paper. */
+  if ((cinfo.jpeg_color_space == JCS_CMYK) ||
+      (cinfo.jpeg_color_space == JCS_YCCK))
+    {
+      cinfo.out_color_space = JCS_CMYK;
+    }
+  else
+    {
+      cinfo.out_color_space = JCS_RGB;
+    }
 
   if (jpeg_start_decompress (&cinfo) != TRUE)
     fprintf (stderr, "gd-jpeg: warning: jpeg_start_decompress"
@@ -353,8 +375,7 @@ gdImageCreateFromJpegCtx (gdIOCtx * infile)
   printf ("gd-jpeg: JPEG image information:");
   if (cinfo.saw_JFIF_marker)
     printf (" JFIF version %d.%.2d",
-	    (int) cinfo.JFIF_major_version,
-	    (int) cinfo.JFIF_minor_version);
+	    (int) cinfo.JFIF_major_version, (int) cinfo.JFIF_minor_version);
   else if (cinfo.saw_Adobe_marker)
     printf (" Adobe format");
   else
@@ -363,10 +384,8 @@ gdImageCreateFromJpegCtx (gdIOCtx * infile)
   printf (" %ux%u (raw) / %ux%u (scaled) %d-bit", cinfo.image_width,
 	  cinfo.image_height, cinfo.output_width,
 	  cinfo.output_height, cinfo.data_precision);
-  printf (" %s", (cinfo.progressive_mode ? "progressive" :
-		  "baseline"));
-  printf (" image, %d quantized colors, ",
-	  cinfo.actual_number_of_colors);
+  printf (" %s", (cinfo.progressive_mode ? "progressive" : "baseline"));
+  printf (" image, %d quantized colors, ", cinfo.actual_number_of_colors);
 
   switch (cinfo.jpeg_color_space)
     {
@@ -409,25 +428,57 @@ gdImageCreateFromJpegCtx (gdIOCtx * infile)
 #if 0
   gdImageInterlace (im, cinfo.progressive_mode != 0);
 #endif
-  if (cinfo.output_components != 3)
+  if (cinfo.out_color_space == JCS_RGB)
     {
-      fprintf (stderr, "gd-jpeg: error: JPEG color quantization"
-	       " request resulted in output_components == %d"
-	       " (expected 3)\n", cinfo.output_components);
+      if (cinfo.output_components != 3)
+	{
+	  fprintf (stderr, "gd-jpeg: error: JPEG color quantization"
+		   " request resulted in output_components == %d"
+		   " (expected 3 for RGB)\n", cinfo.output_components);
+	  goto error;
+	}
+      channels = 3;
+    }
+  else if (cinfo.out_color_space == JCS_CMYK)
+    {
+      jpeg_saved_marker_ptr marker;
+      if (cinfo.output_components != 4)
+	{
+	  fprintf (stderr, "gd-jpeg: error: JPEG color quantization"
+		   " request resulted in output_components == %d"
+		   " (expected 4 for CMYK)\n", cinfo.output_components);
+	  goto error;
+	}
+      channels = 4;
+      marker = cinfo.marker_list;
+      while (marker)
+	{
+	  if ((marker->marker == (JPEG_APP0 + 14)) &&
+	      (marker->data_length >= 12)
+	      && (!strncmp ((const char *) marker->data, "Adobe", 5)))
+	    {
+	      inverted = 1;
+	      break;
+	    }
+	  marker = marker->next;
+	}
+    }
+  else
+    {
+      fprintf (stderr, "gd-jpeg: error: unexpected colorspace\n");
       goto error;
     }
-
 #if BITS_IN_JSAMPLE == 12
   fprintf (stderr, "gd-jpeg: error: jpeg library was compiled for 12-bit\n"
-	 "precision. This is mostly useless, because JPEGs on the web are\n"
-	 "8-bit and such versions of the jpeg library won't read or write\n"
+	   "precision. This is mostly useless, because JPEGs on the web are\n"
+	   "8-bit and such versions of the jpeg library won't read or write\n"
 	   "them. GD doesn't support these unusual images. Edit your\n"
-	 "jmorecfg.h file to specify the correct precision and completely\n"
+	   "jmorecfg.h file to specify the correct precision and completely\n"
 	   "'make clean' and 'make install' libjpeg again. Sorry.\n");
   goto error;
 #endif /* BITS_IN_JSAMPLE == 12 */
 
-  row = gdCalloc (cinfo.output_width * 3, sizeof (JSAMPLE));
+  row = gdCalloc (cinfo.output_width * channels, sizeof (JSAMPLE));
   if (row == 0)
     {
       fprintf (stderr, "gd-jpeg: error: unable to allocate row for"
@@ -435,26 +486,52 @@ gdImageCreateFromJpegCtx (gdIOCtx * infile)
       goto error;
     }
   rowptr[0] = row;
-
-  for (i = 0; i < cinfo.output_height; i++)
+  if (cinfo.out_color_space == JCS_CMYK)
     {
-      nrows = jpeg_read_scanlines (&cinfo, rowptr, 1);
-      if (nrows != 1)
+      for (i = 0; i < cinfo.output_height; i++)
 	{
-	  fprintf (stderr, "gd-jpeg: error: jpeg_read_scanlines"
-		   " returns %u, expected 1\n", nrows);
-	  goto error;
+	  register JSAMPROW currow = row;
+	  register int *tpix = im->tpixels[i];
+	  nrows = jpeg_read_scanlines (&cinfo, rowptr, 1);
+	  if (nrows != 1)
+	    {
+	      fprintf (stderr, "gd-jpeg: error: jpeg_read_scanlines"
+		       " returns %u, expected 1\n", nrows);
+	      goto error;
+	    }
+	  for (j = 0; j < cinfo.output_width; j++, currow += 4, tpix++)
+	    {
+	      *tpix =
+		CMYKToRGB (currow[0], currow[1], currow[2], currow[3],
+			   inverted);
+	    }
+	}
+    }
+  else
+    {
+      for (i = 0; i < cinfo.output_height; i++)
+	{
+	  register JSAMPROW currow = row;
+	  register int *tpix = im->tpixels[i];
+	  nrows = jpeg_read_scanlines (&cinfo, rowptr, 1);
+	  if (nrows != 1)
+	    {
+	      fprintf (stderr, "gd-jpeg: error: jpeg_read_scanlines"
+		       " returns %u, expected 1\n", nrows);
+	      goto error;
+	    }
+	  for (j = 0; j < cinfo.output_width; j++, currow += 3, tpix++)
+	    {
+	      *tpix = gdTrueColor (currow[0], currow[1], currow[2]);
+	    }
 	}
-
-      for (j = 0; j < cinfo.output_width; j++)
-	im->tpixels[i][j] = gdTrueColor (row[j * 3], row[j * 3 + 1],
-					 row[j * 3 + 2]);
     }
-
   if (jpeg_finish_decompress (&cinfo) != TRUE)
     fprintf (stderr, "gd-jpeg: warning: jpeg_finish_decompress"
 	     " reports suspended data source\n");
-
+  /* Thanks to Truxton Fulton */
+  if (cinfo.err->num_warnings > 0)
+    goto error;
 
   jpeg_destroy_decompress (&cinfo);
   gdFree (row);
@@ -469,6 +546,63 @@ error:
   return 0;
 }
 
+/* A very basic conversion approach, TBB */
+
+static int
+CMYKToRGB (int c, int m, int y, int k, int inverted)
+{
+  if (inverted)
+    {
+      c = 255 - c;
+      m = 255 - m;
+      y = 255 - y;
+      k = 255 - k;
+    }
+  return gdTrueColor ((255 - c) * (255 - k) / 255,
+		      (255 - m) * (255 - k) / 255,
+		      (255 - y) * (255 - k) / 255);
+#if 0
+  if (inverted)
+    {
+      c = 255 - c;
+      m = 255 - m;
+      y = 255 - y;
+      k = 255 - k;
+    }
+  c = c * (255 - k) / 255 + k;
+  if (c > 255)
+    {
+      c = 255;
+    }
+  if (c < 0)
+    {
+      c = 0;
+    }
+  m = m * (255 - k) / 255 + k;
+  if (m > 255)
+    {
+      m = 255;
+    }
+  if (m < 0)
+    {
+      m = 0;
+    }
+  y = y * (255 - k) / 255 + k;
+  if (y > 255)
+    {
+      y = 255;
+    }
+  if (y < 0)
+    {
+      y = 0;
+    }
+  c = 255 - c;
+  m = 255 - m;
+  y = 255 - y;
+  return gdTrueColor (c, m, y);
+#endif
+}
+
 /*
 
  * gdIOCtx JPEG data sources and sinks, T. Boutell
@@ -492,14 +626,14 @@ typedef int safeboolean;
 /* Expanded data source object for gdIOCtx input */
 
 typedef struct
-  {
-    struct jpeg_source_mgr pub;	/* public fields */
+{
+  struct jpeg_source_mgr pub;	/* public fields */
 
-    gdIOCtx *infile;		/* source stream */
-    unsigned char *buffer;	/* start of buffer */
-    safeboolean start_of_file;	/* have we gotten any data yet? */
-     
-  }
+  gdIOCtx *infile;		/* source stream */
+  unsigned char *buffer;	/* start of buffer */
+  safeboolean start_of_file;	/* have we gotten any data yet? */
+
+}
 my_source_mgr;
 
 typedef my_source_mgr *my_src_ptr;
@@ -558,45 +692,44 @@ init_source (j_decompress_ptr cinfo)
  */
 
 #define END_JPEG_SEQUENCE "\r\n[*]--:END JPEG:--[*]\r\n"
-
 safeboolean
 fill_input_buffer (j_decompress_ptr cinfo)
 {
   my_src_ptr src = (my_src_ptr) cinfo->src;
-  size_t nbytes = 0;
-  
-  /* size_t got; */
-  /* char *s; */
-    memset (src->buffer, 0, INPUT_BUF_SIZE);
-  
-    while (nbytes < INPUT_BUF_SIZE)
+  /* 2.0.12: signed size. Thanks to Geert Jansen */
+  /* 2.0.14: some platforms (mingw-msys) don't have ssize_t. Call 
+     an int an int. */
+  int nbytes = 0;
+  memset (src->buffer, 0, INPUT_BUF_SIZE);
+
+  while (nbytes < INPUT_BUF_SIZE)
     {
-      
-	int got = gdGetBuf (src->buffer + nbytes, 
-			    INPUT_BUF_SIZE - nbytes,
-			    src->infile);
-      
-	if ((got == EOF) || (got == 0))
+
+      int got = gdGetBuf (src->buffer + nbytes,
+			  INPUT_BUF_SIZE - nbytes,
+			  src->infile);
+
+      if ((got == EOF) || (got == 0))
 	{
-	  
+
 	  /* EOF or error. If we got any data, don't worry about it.
-	     If we didn't, then this is unexpected. */ 
-	    if (!nbytes)
+	     If we didn't, then this is unexpected. */
+	  if (!nbytes)
 	    {
-	      
-		nbytes = -1;
-	      
+
+	      nbytes = -1;
+
 	    }
-	  
-	    break;
-	  
+
+	  break;
+
 	}
-      
-	nbytes += got;
-      
+
+      nbytes += got;
+
     }
-  
-    if (nbytes <= 0)
+
+  if (nbytes <= 0)
     {
       if (src->start_of_file)	/* Treat empty input file as fatal error */
 	ERREXIT (cinfo, JERR_INPUT_EMPTY);
@@ -672,11 +805,11 @@ skip_input_data (j_decompress_ptr cinfo, long num_bytes)
 void
 term_source (j_decompress_ptr cinfo)
 {
-  
+
 #if 0
 /* never used */
-    my_src_ptr src = (my_src_ptr) cinfo->src;
-  
+  my_src_ptr src = (my_src_ptr) cinfo->src;
+
 #endif
 }
 
@@ -688,8 +821,7 @@ term_source (j_decompress_ptr cinfo)
  */
 
 void
-jpeg_gdIOCtx_src (j_decompress_ptr cinfo,
-		  gdIOCtx * infile)
+jpeg_gdIOCtx_src (j_decompress_ptr cinfo, gdIOCtx * infile)
 {
   my_src_ptr src;
 
@@ -709,7 +841,7 @@ jpeg_gdIOCtx_src (j_decompress_ptr cinfo,
       src->buffer = (unsigned char *)
 	(*cinfo->mem->alloc_small) ((j_common_ptr) cinfo, JPOOL_PERMANENT,
 				    INPUT_BUF_SIZE * sizeof (unsigned char));
-      
+
     }
 
   src = (my_src_ptr) cinfo->src;
