@@ -7,7 +7,6 @@
     with AT&T, you have an infringing copy of this software and cannot use
     it without violating AT&T's intellectual property rights.
 */
-#pragma prototyped
 
 #define EXTERN
 #include "libgraph.h"
@@ -110,8 +109,8 @@ static Agsym_t *dcl_attr(void* obj, char* name, char* value)
 	rv = agfindattr(obj, name);
     if (rv) {
         if (strcmp(rv->value,value)) {
-			fprintf(stderr,"%s %s %s\n",name,value,rv->value);
-            agerror("%s may not be redefined",name);
+			agerr (AGWARN, "Attribute %s=\"%s\" cannot be redeclared as \"%s\"\n",
+				name, rv->value, value);
         }
 		return rv;
     }
@@ -254,7 +253,7 @@ void aginitlib(int gs, int ns, int es)
 	}
 	else
 		if ((AG.graph_nbytes!=gs)||(AG.node_nbytes!=ns)||(AG.edge_nbytes!=es))
-			fprintf(stderr,"aginit() called multiply with inconsistent args\n");
+			agerr(AGWARN,"aginit() called multiply with inconsistent args\n");
 }
 
 char *agget(void* obj, char* attr)
@@ -262,9 +261,9 @@ char *agget(void* obj, char* attr)
 	return agxget(obj,agindex(obj,attr));
 }
 
-void agset(void* obj, char* attr, char* value)
+int agset(void* obj, char* attr, char* value)
 {
-	agxset(obj,agindex(obj,attr),value);
+	return agxset(obj,agindex(obj,attr),value);
 }
 
 int agindex(void* obj, char* name)
@@ -283,12 +282,15 @@ char *agxget(void* obj, int index)
 	return NULL;
 }
 
-void agxset(void* obj, int index, char* buf)
+int agxset(void* obj, int index, char* buf)
 {
 	char	**p;
 	if (index >= 0) {
 		p = ((Agraph_t*)obj)->attr;
 		agstrfree(p[index]);
 		p[index] = agstrdup(buf);
+		return 0;
 	}
+	else return -1;
 }
+
