@@ -7,7 +7,6 @@
     with AT&T, you have an infringing copy of this software and cannot use
     it without violating AT&T's intellectual property rights.
 */
-#pragma prototyped
 
 #include "libgraph.h"
 
@@ -186,8 +185,8 @@ static Agraph_t *agNEWgraph(char* name, Agraph_t* parent, int kind)
 	Agraph_t		*g;
 
 	if (AG.init_called == FALSE) {
-		fprintf(stderr,"libag fatal error -- aginit() was not called\n");
-		exit(1);
+		agerr (AGERR, "libag error -- aginit() was not called\n");
+		return 0;
 	}
 	g			= (Agraph_t*) calloc(1,AG.graph_nbytes);
 	g->tag		= TAG_GRAPH;
@@ -263,6 +262,7 @@ Agraph_t *agopen(char* name, int kind)
 
 	g = agNEWgraph(name,NULL,kind);
 	meta = agNEWgraph(name,NULL,AGMETAGRAPH);
+	if (!g || !meta) return 0;
 	agnodeattr(meta, "agusergraph", NULL);
 	g->meta_node = agnode(meta,name);
 	g->meta_node->attr[0] = (char*) g;
@@ -279,6 +279,7 @@ Agraph_t *agsubg(Agraph_t* g, char* name)
 	if (n) subg = agusergraph(n);
 	else {
 		subg = agNEWgraph(name,g,g->kind);
+		if (!subg) return 0;
 		n = agnode(meta,name);
 		subg->meta_node = n;
 		n->attr[0] = (char*) subg;
