@@ -97,7 +97,7 @@ int agerrors(void)
 char *_agstrcanon(char* arg, char* buf, char* chk)
 {
 	char	*s = arg;
-	unsigned char	uc = *(unsigned char*)s++;
+	unsigned char	uc;
 	char	*p = buf;
 	int		cnt = 0;
 	int		has_special = FALSE;
@@ -112,6 +112,7 @@ char *_agstrcanon(char* arg, char* buf, char* chk)
 	}
 	if (ISEMPTYSTR(arg)) return "\"\"";
 	*p++ = '\"';
+	uc = *(unsigned char*)s++;
 	maybe_num = (isdigit(uc) || (uc == '.'));
 	while (uc) {
 		if (uc == '\"') { *p++ = '\\'; has_special = TRUE; }
@@ -254,26 +255,44 @@ void agwrnode(Agraph_t* g, FILE* fp, Agnode_t* n, int full, int indent)
 	}
 }
 
-static void writenodeandport(FILE *fp, char *node, char *port)
+/* writenodeandport:
+ * We still allow ports to begin with ':', though this is deprecated.
+ * This can only happen now by the user supplying a headport or tailport
+ * attribute. 
+ */
+static void writenodeandport(FILE *fp, char *node, char *port, char* sport)
 {
+	char*  s;
 	fprintf(fp,"%s",canonical(node));	/* slimey i know*/
-	if (port && *port) 
-		fprintf(fp,"%c%s",port[0], _agstrcanon(port+1,getoutputbuffer(port+1),port));
+	if (port && *port) { 
+		if (*port == ':') s = port+1;
+		else s = port;
+		fprintf(fp,":%s", _agstrcanon(s,getoutputbuffer(s),port));
+		if (sport && *sport) 
+			fprintf(fp,":%s", _agstrcanon(sport,getoutputbuffer(sport),sport));
+	}
 }
 
 void agwredge(Agraph_t* g, FILE* fp, Agedge_t* e, int list_all)
 {
-	char		*myval,*defval,*edgeop,*tport,*hport;
+	char		*myval,*defval,*edgeop,*tport,*hport,*tsport,*hsport;
 	int			i,nprint = 0;
 	Agdict_t	*d = e->tail->graph->univ->edgeattr;
 	Agsym_t	*a;
 
-	if (e->attr) {tport = e->attr[TAILX]; hport = e->attr[HEADX];}
-	else tport = hport = "";
+	if (e->attr) {
+		tport = e->attr[TAILX]; 
+		hport = e->attr[HEADX];
+		if (tport[0]) tsport = agget (e, TAILSUB_ID); 
+		else tsport = NULL;
+		if (hport[0]) hsport = agget (e, HEADSUB_ID); 
+		else hsport = NULL;
+	}
+	else tport = hport = tsport = hsport = "";
 	if (g->kind & AGFLAG_DIRECTED) edgeop = "->"; else edgeop = "--";
-	writenodeandport(fp,e->tail->name,tport);
+	writenodeandport(fp,e->tail->name,tport,tsport);
 	fprintf(fp," %s ",edgeop);
-	writenodeandport(fp,e->head->name,hport);
+	writenodeandport(fp,e->head->name,hport,hsport);
 	if (list_all) {
 		for (i = 0; i < dtsize(d->dict); i++) {
 			a = d->list[i];
