@@ -1,4 +1,3 @@
-#pragma prototyped
 
 #include "common.h"
 #include "mem.h"
@@ -16,7 +15,7 @@ int inattrstmt;
 
 static graphframe_t *gstack, *topgframe;
 static Tobj allgraphs, alledges, allnodes;
-static Tobj gdict, edict, ndict, N;
+static Tobj gdict, /* edict, */ ndict, N;
 static long newgid, neweid, newnid, gmark, errflag;
 
 static jmp_buf ljbuf;
@@ -133,11 +132,12 @@ static void filllabeltable (Tobj to, int flag) {
                 wflag = FALSE;
             break;
         case '\\':
-            if (*(lsp + 1))
+            if (*(lsp + 1)) {
                 if (ISCTRL (*(lsp + 1)))
                     lsp++;
                 else if (*(lsp + 1) == ' ')
                     ishardspace = TRUE, lsp++;
+	    }
             /* falling through ... */
         default:
             if ((mode & HASTABLE) && *lsp != ' ')
@@ -250,7 +250,7 @@ void D2Lwritegraph (int ioi, Tobj graph, int flag) {
     Tobj so, no, eo, to;
     char buf[10240];
     char *s;
-    int isdag, n, nn, i;
+    int isdag, n = 0, nn, i;
 
     if (!(so = Tfinds (graph, "type")) || !T_ISSTRING (so))
         s = "digraph";
@@ -448,22 +448,26 @@ static void writeattr (int ioi, Tobj to, char *buf) {
     for (Tgetfirst (to, &tkvi); tkvi.kvp; Tgetnext (&tkvi)) {
         switch (Tgettype (tkvi.kvp->ko)) {
         case T_INTEGER:
-            sprintf (s1, "%d = ", Tgetinteger (tkvi.kvp->ko));
+            sprintf (s1, "%ld = ", Tgetinteger (tkvi.kvp->ko));
             break;
         case T_REAL:
-            sprintf (s1, "%lf = ", Tgetreal (tkvi.kvp->ko));
+            sprintf (s1, "%f = ", Tgetreal (tkvi.kvp->ko));
             break;
         case T_STRING:
             sprintf (s1, "%s = ", Tgetstring (tkvi.kvp->ko));
             break;
+	case T_CODE:
+	case T_TABLE:
+	case T_SIZE:
+	    break;
         }
         s2 = buf + strlen (buf);
         switch (Tgettype (tkvi.kvp->vo)) {
         case T_INTEGER:
-            sprintf (s2, "\"%d\"", Tgetinteger (tkvi.kvp->vo));
+            sprintf (s2, "\"%ld\"", Tgetinteger (tkvi.kvp->vo));
             break;
         case T_REAL:
-            sprintf (s2, "\"%lf\"", Tgetreal (tkvi.kvp->vo));
+            sprintf (s2, "\"%f\"", Tgetreal (tkvi.kvp->vo));
             break;
         case T_STRING:
             *s2++ = '"';
@@ -488,12 +492,14 @@ static void quotestring (char *buf, Tobj so) {
 
     s1 = buf + strlen (buf);
     *s1++ = '"';
-    if (so && T_ISSTRING (so))
-        for (s2 = Tgetstring (so); *s2; s2++)
+    if (so && T_ISSTRING (so)) {
+        for (s2 = Tgetstring (so); *s2; s2++) {
             if (*s2 == '"')
                 *s1++ = '\\', *s1++ = *s2;
             else
                 *s1++ = *s2;
+	}
+    }
     *s1++ = '"', *s1 = 0;
 }
 
@@ -515,7 +521,7 @@ void D2Lbegin (char *name) {
     /* the dictionaries */
     Tinss (gstack->g, "graphdict", (gdict = Ttable (10)));
     Tinss (gstack->g, "nodedict", (ndict = Ttable (10)));
-    Tinss (gstack->g, "edgedict", (edict = Ttable (10)));
+    Tinss (gstack->g, "edgedict", (/* edict = */  Ttable (10)));
 
     /* this graph's nodes, edges, subgraphs */
     Tinss (gstack->g, "nodes", (allnodes = gstack->nodes = Ttable (10)));
