@@ -1,3 +1,7 @@
+#ifdef __cplusplus
+extern "C" {
+#endif
+
 /*
     This software may only be used by you under license from AT&T Corp.
     ("AT&T").  A copy of AT&T's Source Code Agreement is available at
@@ -8,7 +12,6 @@
     it without violating AT&T's intellectual property rights.
 */
 
-#pragma prototyped
 /* Lefteris Koutsofios - AT&T Bell Laboratories */
 
 #ifndef _EXEC_H
@@ -25,3 +28,7 @@ void Eterm (void);
 Tobj Eunit (Tobj);
 Tobj Efunction (Tobj, char *);
 #endif /* _EXEC_H */
+
+#ifdef __cplusplus
+}
+#endif
