@@ -8,7 +8,6 @@
     it without violating AT&T's intellectual property rights.
 */
 
-#pragma prototyped
 /* Lefteris Koutsofios - AT&T Bell Laboratories */
 
 #include "common.h"
@@ -31,6 +30,10 @@
 #include "internal.h"
 #ifndef FEATURE_MS
 #include <sys/time.h>
+#ifdef HAVE_UNISTD_H
+#include <sys/types.h>
+#include <unistd.h>
+#endif
 #endif
 
 int Idump (int, Tonm_t *);
@@ -477,6 +480,10 @@ int Iconcat (int argc, lvar_t *argv) {
             for (s = buf2; *s; s++)
                 bufp[bufi++] = *s;
             break;
+	case T_CODE:
+	case T_TABLE:
+	case T_SIZE:
+	    break;
         }
     }
     bufp[bufi] = '\000';
@@ -507,6 +514,10 @@ int Iquote (int argc, lvar_t *argv) {
         sprintf (buf2, "%f", Tgetreal (so));
         s = &buf2[0];
         break;
+    case T_CODE:
+    case T_TABLE:
+    case T_SIZE:
+        break;
     }
     if (argc > 1)
         qs = Tgetstring (qo);
@@ -674,6 +685,8 @@ int Iecho (int argc, lvar_t *argv) {
         case T_INTEGER: printf ("%ld", Tgetinteger (argv[i].o)); break;
         case T_REAL:    printf ("%f", Tgetreal (argv[i].o));     break;
         case T_TABLE:   printf ("[\n"), Dtrace (argv[i].o, 4);   break;
+	case T_CODE:
+	case T_SIZE: break;
         }
     }
     printf ("\n");
