@@ -1,3 +1,7 @@
+#ifdef __cplusplus
+extern "C" {
+#endif
+
 /*
     This software may only be used by you under license from AT&T Corp.
     ("AT&T").  A copy of AT&T's Source Code Agreement is available at
@@ -8,7 +12,6 @@
     it without violating AT&T's intellectual property rights.
 */
 
-#pragma prototyped
 /* Lefteris Koutsofios - AT&T Bell Laboratories */
 
 #ifndef _STR_H
@@ -22,3 +25,7 @@ char *Stfull (Tobj);
 char *Ssfull (Tobj, Tobj);
 char *Scfull (Tobj, int, int);
 #endif /* _STR_H */
+
+#ifdef __cplusplus
+}
+#endif
