@@ -8,7 +8,6 @@
     it without violating AT&T's intellectual property rights.
 */
 
-#pragma prototyped
 /* Lefteris Koutsofios - AT&T Bell Laboratories */
 
 #include "common.h"
@@ -331,7 +330,7 @@ void Tdelo (Tobj to, Tobj ko) {
 }
 
 Tobj Tcopy (Tobj fmvo) {
-    Tobj tovo;
+    Tobj tovo=NULL;
     long m;
 
     switch (M_TYPEOF (fmvo)) {
@@ -381,13 +380,13 @@ void Tgetnext (Tkvindex_t *p) {
 }
 
 static void insert (Ttable_t *tp, Tobj ko, char *sk, Tobj vo) {
-    Tkvlist_t *kvlp, *nkvlp;
+    Tkvlist_t *kvlp=NULL, *nkvlp;
     Tkv_t *kvp;
     Ttype_t kt;
-    long ik, i, ind, nln;
-    double rk;
+    long ik = 0, i, ind = 0, nln;
+    double rk = 0.0;
 
-    switch ((kt = M_TYPEOF (ko))) {
+    switch ((kt = (Ttype_t)M_TYPEOF (ko))) {
     case T_INTEGER:
         ik = ((Tinteger_t *) ko)->i;
         if ((kvlp = tp->lp[(ind = GETIKINDEX (tp, ik))]))
@@ -410,6 +409,8 @@ static void insert (Ttable_t *tp, Tobj ko, char *sk, Tobj vo) {
                 if (ISEQSK (sk, kvp[i].ko))
                     goto found;
         break;
+    default:
+	break;
     }
     if ((nln = tp->n + 1) > 4 * tp->ln && nln < M_SIZEMAX) {
         copytable (tp, nln);
@@ -417,6 +418,7 @@ static void insert (Ttable_t *tp, Tobj ko, char *sk, Tobj vo) {
         case T_INTEGER: ind = GETIKINDEX (tp, ik); break;
         case T_REAL:    ind = GETRKINDEX (tp, rk); break;
         case T_STRING:  ind = GETSKINDEX (tp, sk); break;
+	default: break;
         }
         kvlp = tp->lp[ind];
     }
@@ -436,6 +438,7 @@ static void insert (Ttable_t *tp, Tobj ko, char *sk, Tobj vo) {
         case T_INTEGER: ko = Tinteger (ik); break;
         case T_REAL:    ko = Treal (rk);    break;
         case T_STRING:  ko = Tstring (sk);  break;
+	default: break;
         }
     }
 
@@ -473,6 +476,7 @@ static Tobj find (Ttable_t *tp, Tobj ko, char *sk) {
     case T_STRING:
         if (M_AREAOF (ko) != 0)
             sk = ((Tstring_t *) ko)->s;
+		if (sk  == NULL) return NULL;
         if ((kvlp = tp->lp[GETSKINDEX (tp, sk)]))
             for (i = 0, kvp = &kvlp->kv[0]; i < kvlp->i; i++)
                 if (ISEQSK (sk, kvp[i].ko))
@@ -527,9 +531,9 @@ found:
 
 static void copytable (Ttable_t *tp, long ln) {
     Tkvlist_t **olp, **lp;
-    Tkvlist_t *okvlp, *kvlp, *nkvlp;
+    Tkvlist_t *okvlp, *kvlp=NULL, *nkvlp;
     Tkv_t *kvp;
-    long ik, oln, i, j, k, ind;
+    long ik, oln, i, j, k, ind = 0;
     double rk;
     char *sk;
 
