@@ -1,3 +1,7 @@
+#ifdef __cplusplus
+extern "C" {
+#endif
+
 /*
     This software may only be used by you under license from AT&T Corp.
     ("AT&T").  A copy of AT&T's Source Code Agreement is available at
@@ -8,7 +12,6 @@
     it without violating AT&T's intellectual property rights.
 */
 
-#pragma prototyped
 /* Lefteris Koutsofios - AT&T Bell Laboratories */
 
 #ifndef _TXTVIEW_H
@@ -21,3 +24,7 @@ void TXTprocess (int, char *);
 void TXTupdate (void);
 void TXTtoggle (int, void *);
 #endif /* _TXTVIEW_H */
+
+#ifdef __cplusplus
+}
+#endif
