@@ -1,4 +1,3 @@
-#pragma prototyped
 /* Lefteris Koutsofios - AT&T Bell Laboratories */
 
 #include "common.h"
@@ -450,7 +449,7 @@ int GCgetwidgetattr (Gwidget_t *widget, int attrn, Gwattr_t *attrp) {
             attrp[ai].u.t = &Gbufp[0];
             break;
         case G_ATTREVENTCB:
-            attrp[ai].u.func = WCU->func;
+            attrp[ai].u.func = (void*)(WCU->func);
             break;
         case G_ATTRUSERDATA:
             attrp[ai].u.u = widget->udata;
@@ -721,7 +720,7 @@ int GCtext (Gwidget_t *widget, Gtextline_t *tlp, int n, Gpoint_t go,
     PIXrect_t pr;
     Grect_t gr;
     XFontStruct *font;
-    int dir, asc, des, x, y, w, h, i;
+    int dir, asc, des, x = 0, y, w, h, i;
     XCharStruct txtinfo;
 
     po = pdrawtopix (widget, go);
@@ -869,7 +868,7 @@ int GCdestroybitmap (Gbitmap_t *bitmap) {
 #define CMINMAXDIFF 20000
 
 int GCreadbitmap (Gwidget_t *widget, Gbitmap_t *bitmap, FILE *fp) {
-    Gsize_t s;
+    Gsize_t s = {0,0};
     XImage *img;
     XColor colors[G_MAXCOLORS];
     char bufp[2048];
@@ -1122,8 +1121,8 @@ static int scalebitmap (Gwidget_t *widget, Gbitmap_t *bitmap, Gsize_t scale,
     Pixmap spix;
     XImage *oimg, *simg;
     XColor colors[G_MAXCOLORS];
-    int cmaxdiff, colorn, colori, x, y, x2, y2, xp, yp;
-    double prod, rgb[3], xr2, yr2, xl2, yl2, xf2, yf2, xr, yr, xl, yl;
+    int cmaxdiff, colorn, colori, x, y, x2, y2 = 0, xp, yp;
+    double prod, rgb[3], xr2, yr2 = 0, xl2, yl2 = 0, xf2, yf2, xr, yr, xl, yl;
 
     if (!copybits) {
         if (dir == 1) {
