@@ -5,7 +5,7 @@
  */
 
 #ifdef HAVE_CONFIG_H
-#include "gvconfig.h"
+#include "config.h"
 #endif
 
 #ifdef HAVE_VALUES_H
@@ -29,6 +29,8 @@ static int NS_run;
 #include <agraph.h>
 #include <agutil.h>
 #include <stdio.h>
+#include <stdlib.h>
+#include <string.h>
 
 #include "nspvt.h"
 
@@ -120,7 +122,9 @@ static char NS_graphdata[] = "ns_graphdata";
 /* forward declarations */
 void init_cutvalues(Agraph_t *g);
 static void ns_check_graph(Agraph_t *g);
+#if 0 /* not used */
 static void ns_check_cutvalues(Agraph_t *g);
+#endif
 static void ns_attachattrs(Agraph_t *g);
 static void ns_normalize(Agraph_t *g);
 static int  ns_check_ranks(Agraph_t *g, int iter, int verbose);
@@ -129,11 +133,11 @@ static int  ns_check_ranks(Agraph_t *g, int iter, int verbose);
 static boolean descriptor_to_front(void *obj, char *id, int sz)
 {
 	boolean		rv;
-	Agrec_t		*rec;
+	/* Agrec_t		*rec; */
 
-	if ((rec = aggetrec(obj, id, TRUE))) rv = TRUE;
+	if ((/* rec = */ aggetrec(obj, id, TRUE))) rv = TRUE;
 	else {
-		rec = agbindrec(obj, id, sz, TRUE);
+		/* rec = */ agbindrec(obj, id, sz, TRUE);
 		rv = FALSE;
 	}
 	return rv;
@@ -335,7 +339,7 @@ Agedge_t *leave_edge(Agraph_t *g)
 {
 	Agnode_t		*n, *start;
 	Agedge_t		*e, *rv = NULL;
-	int				cnt, cv, pcv;
+	int				cnt, cv, pcv = 0;
 
 	start = finger(g);
 	if (start == NILnode) start = agfstn(g);
@@ -744,6 +748,7 @@ static int ns_check_ranks(Agraph_t *g, int iter, int verbose)
 	return cost;
 }
 
+#if 0 /* not used */
 static void ns_check_cutvalues(Agraph_t *g)
 {
 	Agnode_t	*v;
@@ -763,6 +768,7 @@ static void ns_check_cutvalues(Agraph_t *g)
 		}
 	}
 }
+#endif
 
 static void ns_checkdfs(Agnode_t *n)
 {
@@ -789,12 +795,10 @@ static void ns_checkreach(Agnode_t *n)
 {
 	Agedge_t	*e;
 
-Agnode_t *bum;
 	if (n) {
 		dmark(n) = TRUE;
 		for (e = agfstedge(n); e; e = agnxtedge(e,n)) {
-			bum = e->node;
-			assert(agsubnode(agraphof(bum),bum,FALSE));
+			assert(agsubnode(agraphof(e->node),e->node,FALSE));
 			if (dmark(e->node) == FALSE) ns_checkreach(e->node);
 		}
 	}
