@@ -7,11 +7,12 @@
     with AT&T, you have an infringing copy of this software and cannot use
     it without violating AT&T's intellectual property rights.
 */
-#pragma prototyped
 
 #include <stdlib.h>
 #include <stdio.h>
+#ifdef HAVE_MALLOC_H
 #include <malloc.h>
+#endif
 #include <math.h>
 #include "pathplan.h"
 #include "solvers.h"
@@ -20,8 +21,8 @@
 #include "dmalloc.h"
 #endif
 
-#define EPSILON1 1E-6
-#define EPSILON2 1E-10
+#define EPSILON1 1E-3
+#define EPSILON2 1E-6
 
 #define ABS(a) ((a) >= 0 ? (a) : -(a))
 
@@ -251,7 +252,7 @@ static int mkspline (Ppoint_t *inps, int inpn, tna_t *tnas, Ppoint_t ev0, Ppoint
     det01 = c[0][0] * c[1][1] - c[1][0] * c[0][1];
     det0X = c[0][0] * x[1] - c[0][1] * x[0];
     detX1 = x[0] * c[1][1] - x[1] * c[0][1];
-    if (det01 != 0.0) {
+    if (ABS(det01) >= 1e-6) {
         scale0 = detX1 / det01;
         scale3 = det0X / det01;
     }
