@@ -18,6 +18,10 @@
 #include <vispath.h>
 #include <pathutil.h>
 
+#ifdef __cplusplus
+extern "C" {
+#endif
+
 typedef COORD** array2;
 typedef	unsigned char boolean;
 
@@ -42,9 +46,13 @@ struct vconfig_s {
 
 extern COORD* ptVis (vconfig_t *, int, Ppoint_t);
 extern int directVis (Ppoint_t, int, Ppoint_t, int, vconfig_t *);
-extern void vis (vconfig_t *);
+extern void visibility (vconfig_t *);
 extern int* makePath (Ppoint_t p, int pp, COORD* pvis,
                Ppoint_t q, int qp, COORD* qvis,
                vconfig_t* conf);
 
+#ifdef __cplusplus
+}
+#endif
+
 #endif
