@@ -1,4 +1,7 @@
-#pragma prototyped
+#ifdef __cplusplus
+extern "C" {
+#endif
+
 /*
  * standalone mini error interface
  */
@@ -12,6 +15,7 @@
 typedef struct Error_info_s
 {
 	int		errors;
+	int		indent;
 	int		line;
 	int		warnings;
 	char*		file;
@@ -43,3 +47,7 @@ extern void	errorf(void*, void*, int, ...);
 extern void	errorv(const char*, int, va_list);
 
 #endif
+
+#ifdef __cplusplus
+}
+#endif
