@@ -1,4 +1,3 @@
-#pragma prototyped
 
 #include <ast.h>
 
@@ -10,6 +9,6 @@ char*
 strcopy(register char* s, register const char* t)
 {
 	if (!t) return(s);
-	while (*s++ = *t++);
+	while ((*s++ = *t++));
 	return(--s);
 }
