@@ -1,4 +1,3 @@
-#pragma prototyped
 /*
  * Glenn Fowler
  * AT&T Research
@@ -25,8 +24,8 @@ static void		gen(Excc_t*, Exnode_t*);
  * return C name for op
  */
 
-static char*
-opname(int op)
+char*
+exopname(int op)
 {
 	static char	buf[16];
 
@@ -83,7 +82,7 @@ opname(int op)
 	case RS:
 		return ">>";
 	}
-	sfsprintf(buf, sizeof(buf) - 1, "(OP=%03o)", op);
+	sfsprintf(buf, sizeof(buf) - 1, "(OP=%03d)", op);
 	return buf;
 }
 
@@ -122,6 +121,41 @@ print(Excc_t* cc, Exnode_t* expr)
 	}
 }
 
+/*
+ * generate scanf()
+ */
+
+static void
+scan(Excc_t* cc, Exnode_t* expr)
+{
+	register Print_t*	x;
+	register int		i;
+
+	if ((x = expr->data.print.args))
+	{
+		sfprintf(cc->ccdisc->text, "sfscanf(sfstdin, \"%s", fmtesq(x->format, quote));
+		while ((x = x->next))
+			sfprintf(cc->ccdisc->text, "%s", fmtesq(x->format, quote));
+		sfprintf(cc->ccdisc->text, "\"");
+		for (x = expr->data.print.args; x; x = x->next)
+		{
+			if (x->arg)
+			{
+				for (i = 0; i < elementsof(x->param) && x->param[i]; i++)
+				{
+					sfprintf(cc->ccdisc->text, ", &(");
+					gen(cc, x->param[i]);
+					sfprintf(cc->ccdisc->text, ")");
+				}
+				sfprintf(cc->ccdisc->text, ", &(");
+				gen(cc, x->arg);
+				sfprintf(cc->ccdisc->text, ")");
+			}
+		}
+		sfprintf(cc->ccdisc->text, ");\n");
+	}
+}
+
 /*
  * internal excc
  */
@@ -183,6 +217,32 @@ gen(Excc_t* cc, register Exnode_t* expr)
 		gen(cc, x);
 		sfprintf(cc->ccdisc->text, ");\n");
 		return;
+	case RAND:
+		sfprintf(cc->ccdisc->text, "rand();\n");
+		return;
+	case SRAND:
+		if (expr->binary) {
+			sfprintf(cc->ccdisc->text, "srand(");
+			gen(cc, x);
+			sfprintf(cc->ccdisc->text, ");\n");
+		}
+		else
+			sfprintf(cc->ccdisc->text, "srand();\n");
+		return;
+	case GSUB:
+	case SUB:
+	case SUBSTR:
+		s = (expr->op == GSUB ? "gsub(" : expr->op == SUB ? "sub(" : "substr(");
+		sfprintf(cc->ccdisc->text, s);
+    	gen(cc,expr->data.string.base);
+		sfprintf(cc->ccdisc->text, ", ");
+    	gen(cc,expr->data.string.pat);
+    	if (expr->data.string.repl) {
+			sfprintf(cc->ccdisc->text, ", ");
+			gen(cc,expr->data.string.repl);
+		}
+		sfprintf(cc->ccdisc->text, ")");
+		return;
 	case IF:
 		sfprintf(cc->ccdisc->text, "if (");
 		gen(cc, x);
@@ -228,6 +288,11 @@ gen(Excc_t* cc, register Exnode_t* expr)
 			sfprintf(cc->ccdisc->text, "} }");
 		}
 		return;
+	case PRINT:
+		sfprintf(cc->ccdisc->text, "print");
+		if (x) gen(cc, x);
+		else sfprintf(cc->ccdisc->text, "()");
+		return;
 	case PRINTF:
 		print(cc, expr);
 		return;
@@ -236,6 +301,9 @@ gen(Excc_t* cc, register Exnode_t* expr)
 		gen(cc, x);
 		sfprintf(cc->ccdisc->text, ");\n");
 		return;
+	case SCANF:
+		scan(cc, expr);
+		return;
 	case SWITCH:
 		t = x->type;
 		sfprintf(cc->ccdisc->text, "{ %s %stmp_%d = ", extype(t), cc->id, ++cc->tmp);
@@ -303,7 +371,7 @@ gen(Excc_t* cc, register Exnode_t* expr)
 		sfprintf(cc->ccdisc->text, "}");
 		return;
 	case '=':
-		sfprintf(cc->ccdisc->text, "(%s%s=", x->data.variable.symbol->name, expr->subop == '=' ? "" : opname(expr->subop));
+		sfprintf(cc->ccdisc->text, "(%s%s=", x->data.variable.symbol->name, expr->subop == '=' ? "" : exopname(expr->subop));
 		gen(cc, expr->data.operand.right);
 		sfprintf(cc->ccdisc->text, ")");
 		return;
@@ -316,6 +384,7 @@ gen(Excc_t* cc, register Exnode_t* expr)
 				case FOR:
 				case IF:
 				case PRINTF:
+				case PRINT:
 				case RETURN:
 				case WHILE:
 					break;
@@ -334,6 +403,7 @@ gen(Excc_t* cc, register Exnode_t* expr)
 			case FOR:
 			case IF:
 			case PRINTF:
+			case PRINT:
 			case RETURN:
 			case WHILE:
 				break;
@@ -470,12 +540,12 @@ gen(Excc_t* cc, register Exnode_t* expr)
 	else
 	{
 		if (!y)
-			sfprintf(cc->ccdisc->text, "%s", opname(expr->op));
+			sfprintf(cc->ccdisc->text, "%s", exopname(expr->op));
 		sfprintf(cc->ccdisc->text, "(");
 		gen(cc, x);
 		if (y)
 		{
-			sfprintf(cc->ccdisc->text, ")%s(", opname(expr->op));
+			sfprintf(cc->ccdisc->text, ")%s(", exopname(expr->op));
 			gen(cc, y);
 		}
 		sfprintf(cc->ccdisc->text, ")");
