@@ -1,4 +1,3 @@
-#pragma prototyped
 /*
  * Glenn Fowler
  * AT&T Research
@@ -152,24 +151,42 @@ trace(Expr_t* ex, int lev, char* op, int c)
 	case FOR:
 		s = " for";
 		break;
+	case GSUB:
+		s = " gsub";
+		break;
 	case IF:
 		s = " if";
 		break;
 	case PRAGMA:
 		s = " pragma";
 		break;
+	case PRINT:
+		s = " print";
+		break;
 	case PRINTF:
 		s = " printf";
 		break;
 	case QUERY:
 		s = " query";
 		break;
+	case RAND:
+		s = " rand";
+		break;
 	case RETURN:
 		s = " return";
 		break;
 	case SPRINTF:
 		s = " sprintf";
 		break;
+	case SRAND:
+		s = " srand";
+		break;
+	case SUB:
+		s = " sub";
+		break;
+	case SUBSTR:
+		s = " substr";
+		break;
 	case SWITCH:
 		s = " switch";
 		break;
@@ -196,20 +213,20 @@ trace(Expr_t* ex, int lev, char* op, int c)
 }
 
 /*
- * trace wrapper for extoken()
+ * trace wrapper for extoken_fn()
  */
 
 
-int _extoken_(register Expr_t* ex);
+int _extoken_fn_(register Expr_t* ex);
 
 int
-extoken(Expr_t* ex)
+extoken_fn(Expr_t* ex)
 {
 	int	c;
 
-#define extoken	_extoken_
+#define extoken_fn	_extoken_fn_
 
-	c = extoken(ex);
+	c = extoken_fn(ex);
 	trace(ex, 0, "exlex", c);
 	return c;
 }
@@ -292,12 +309,11 @@ lex(register Expr_t* ex)
  */
 
 int
-extoken(register Expr_t* ex)
+extoken_fn(register Expr_t* ex)
 {
 	register int	c;
 	register char*	s;
 	register int	q;
-	int		b = 0;
 	char*		e;
 
 	if (ex->eof || ex->errors)
@@ -431,7 +447,7 @@ extoken(register Expr_t* ex)
 				s--;
 			if (s == ex->line)
 			{
-				switch (extoken(ex))
+				switch (extoken_fn(ex))
 				{
 				case DYNAMIC:
 				case ID:
@@ -444,7 +460,7 @@ extoken(register Expr_t* ex)
 				}
 				if (streq(s, "include"))
 				{
-					if (extoken(ex) != STRING)
+					if (extoken_fn(ex) != STRING)
 						exerror("#%s: string argument expected", s);
 					else if (!expush(ex, exlval.string, 1, NiL, NiL))
 					{
@@ -505,10 +521,8 @@ extoken(register Expr_t* ex)
 		sfstrset(ex->tmp, 0);
 		sfputc(ex->tmp, c);
 		q = INTEGER;
-		b = 0;
 		if ((c = lex(ex)) == 'x' || c == 'X')
 		{
-			b = 16;
 			sfputc(ex->tmp, c);
 			for (;;)
 			{
